@@ -59,11 +59,12 @@ void ClaspAppOptions::initOptions(ProgramOptions::OptionContext& root) {
 	using namespace ProgramOptions;
 	OptionGroup basic("Basic Options");
 	basic.addOptions()
-		("quiet,q"    , notify(this, &ClaspAppOptions::mappedOpts)->implicit("2,2,2")->arg("<m>,<o>"), 
+		("quiet,q"    , notify(this, &ClaspAppOptions::mappedOpts)->implicit("2,2,2")->arg("<levels>"), 
 		 "Configure printing of models, costs, and calls\n"
-		 "      <m>: print {0=all|1=last|2=no} models\n"
-		 "      <o>: print {0=all|1=last|2=no} optimize values [<m>]\n"
-		 "      <c>: print {0=all|1=last|2=no} call steps      [2]")
+		 "      %A: <mod>[,<cost>][,<call>]\n"
+		 "        <mod> : print {0=all|1=last|2=no} models\n"
+		 "        <cost>: print {0=all|1=last|2=no} optimize values [<m>]\n"
+		 "        <call>: print {0=all|1=last|2=no} call steps      [2]")
 		("pre" , flag(onlyPre), "Run preprocessing and exit")
 		("print-portfolio" , flag(printPort), "Print default portfolio and exit")
 		("outf,@1", storeTo(outf)->arg("<n>"), "Use {0=default|1=competition|2=JSON|3=no} output")
@@ -84,7 +85,7 @@ bool ClaspAppOptions::mappedOpts(ClaspAppOptions* this_, const std::string& name
 	if (name == "quiet") {
 		const char* err = 0;
 		uint32      q[3]= {uint32(UCHAR_MAX),uint32(UCHAR_MAX),uint32(UCHAR_MAX)};
-		int      parsed = bk_lib::xconvert(value.c_str(), q, &err, 3);
+		int      parsed = bk_lib::xconvert(value.c_str(), q, &err);
 		for (int i = 0; i != parsed; ++i) { this_->quiet[i] = static_cast<uint8>(q[i]); }
 		return parsed && *err == 0;
 	}
@@ -326,7 +327,7 @@ void ClaspAppBase::printDefaultConfigs() const {
 		printf("%s:\n%*c", it.name(), minW-1, ' ');
 		cmd = it.args();
 		// split options into formatted lines
-		uint32 sz = cmd.size(), off = 0, n = maxW - minW;
+		std::size_t sz = cmd.size(), off = 0, n = maxW - minW;
 		while (n < sz) {
 			while (n != off  && cmd[n] != ' ') { --n; }
 			if (n != off) { cmd[n] = 0; printf("%s\n%*c", &cmd[off], minW-1, ' '); }
@@ -406,38 +407,9 @@ std::istream& ClaspAppBase::getStream() {
 	}
 }
 
-bool ClaspAppBase::updateConfig(const std::string& solvingOptions, bool resetOptions) {
-	using namespace ProgramOptions;
-	claspConfig_.reset();
-	if (resetOptions) { argVec_.clear(); }
-	OptionContext opts;
-	claspConfig_.addOptions(opts);
-	ParsedOptions exclude;
-	ParsedValues  values(ProgramOptions::parseCommandString(solvingOptions, opts, false, 0, ProgramOptions::command_line_allow_flag_value));
-	exclude.assign(values);
-	claspConfig_.addDisabled(exclude);
-	// reassign previous options
-	ArgVec::iterator j = argVec_.begin();
-	for (ArgVec::iterator it = j, end = argVec_.end(); it != end; ++it) {
-		OptionContext::option_iterator opt = opts.tryFind(it->first.c_str());
-		if (opt != opts.end() && (exclude.count(it->first) == 0 || (*opt)->value()->isComposing())) {
-			(*opt)->value()->parse(it->first, it->second, Value::value_unassigned);
-			exclude.add(it->first);
-			*j++ = *it;
-		}
-	}
-	argVec_.erase(j, argVec_.end());
-	// remember new options
-	for (ParsedValues::iterator it = values.begin(), end = values.end(); it != end; ++it) {
-		argVec_.push_back(ArgVal(it->first->name(), it->second));
-	}
-	opts.assignDefaults(exclude);
-	return claspConfig_.finalize(exclude, getProblemType(), true);
-}
-
 // Creates output object suitable for given input format
 Output* ClaspAppBase::createOutput(ProblemType f) {
-	std::auto_ptr<Output> out;
+	SingleOwnerPtr<Output> out;
 	if (claspAppOpts_.outf == ClaspAppOptions::out_none) {
 		return 0;
 	}
@@ -449,6 +421,9 @@ Output* ClaspAppBase::createOutput(ProblemType f) {
 			outFormat = TextOutput::format_aspcomp;
 		}
 		out.reset(new TextOutput(verbose(), outFormat, claspAppOpts_.outAtom.c_str(), claspAppOpts_.ifs));
+		if (claspConfig_.solve.maxSat && f == Problem_t::SAT) {
+			static_cast<TextOutput*>(out.get())->result[TextOutput::res_sat] = "UNKNOWN";
+		}
 	}
 	else {
 		out.reset(new JsonOutput(verbose()));
@@ -467,11 +442,8 @@ Output* ClaspAppBase::createOutput(ProblemType f) {
 	}
 	return out.release();
 }
-void ClaspAppBase::storeCommandArgs(const ProgramOptions::ParsedValues& values) { 
-	argVec_.reserve(std::distance(values.begin(), values.end()));
-	for (ProgramOptions::ParsedValues::iterator it = values.begin(), end = values.end(); it != end; ++it) {
-		argVec_.push_back(ArgVal(it->first->name(), it->second));
-	}
+void ClaspAppBase::storeCommandArgs(const ProgramOptions::ParsedValues&) { 
+	/* We don't need the values */
 }
 
 bool ClaspAppBase::handlePostGroundOptions(ProgramBuilder& prg) {
@@ -486,6 +458,21 @@ bool ClaspAppBase::handlePreSolveOptions(ClaspFacade& clasp) {
 	if (!claspAppOpts_.hccOut.empty()  && clasp.ctx.sccGraph.get()){ writeNonHcfs(*clasp.ctx.sccGraph); }
 	return true;
 }
+void ClaspAppBase::run(ClaspFacade& clasp) {
+	ProblemType     pt    = getProblemType();
+	StreamSource    input(getStream());
+	bool            inc   = pt == Problem_t::ASP && *input == '9';
+	ProgramBuilder& prg   = clasp.start(claspConfig_, pt);
+	if (inc) { inc = clasp.enableProgramUpdates(); }
+	else     { claspConfig_.releaseOptions(); }
+	while (prg.parseProgram(input) && handlePostGroundOptions(prg)) {
+		if (clasp.prepare() && handlePreSolveOptions(clasp)) {
+			clasp.solve();
+		}
+		if (!inc || clasp.result().interrupted() || !input.skipWhite() || *input != '9' || !clasp.update().ok()) { break; }
+		prg.disposeMinimizeConstraint();
+	}
+}
 /////////////////////////////////////////////////////////////////////////////////////////
 // ClaspApp
 /////////////////////////////////////////////////////////////////////////////////////////
@@ -497,17 +484,7 @@ ProblemType ClaspApp::getProblemType() {
 }
 
 void ClaspApp::run(ClaspFacade& clasp) {
-	ProblemType     pt    = getProblemType();
-	StreamSource    input(getStream());
-	bool            inc   = pt == Problem_t::ASP && *input == '9';
-	ProgramBuilder& prg   = clasp.start(claspConfig_, pt, inc);
-	while (prg.parseProgram(input) && handlePostGroundOptions(prg)) {
-		if (clasp.prepare() && handlePreSolveOptions(clasp)) {
-			clasp.solve();
-		}
-		if (!inc || !input.skipWhite() || *input != '9' || !clasp.update().ok()) { break; }
-		prg.disposeMinimizeConstraint();
-	}
+	ClaspAppBase::run(clasp);
 }
 
 void ClaspApp::printHelp(const ProgramOptions::OptionContext& root) {
@@ -516,9 +493,6 @@ void ClaspApp::printHelp(const ProgramOptions::OptionContext& root) {
 	printf("Get help/report bugs via : http://sourceforge.net/projects/potassco/support\n");
 	fflush(stdout);
 }
-void ClaspApp::storeCommandArgs(const ProgramOptions::ParsedValues&) {
-	/* We don't need the values */
-}
 /////////////////////////////////////////////////////////////////////////////////////////
 // WriteLemmas
 /////////////////////////////////////////////////////////////////////////////////////////
