@@ -35,8 +35,7 @@ using namespace Clasp::Cli;
 
 // {{{ declaration of ClingoApp
 
-ClingoApp::ClingoApp() {
-}
+ClingoApp::ClingoApp() { }
 
 static bool parseConst(const std::string& str, std::vector<std::string>& out) {
     out.push_back(str);
@@ -51,7 +50,7 @@ void ClingoApp::initOptions(ProgramOptions::OptionContext& root) {
     OptionGroup gringo("Gringo Options");
     gringo.addOptions()
         ("text"                     , flag(grOpts_.text = false)     , "Print plain text format")
-        ("const,c"                  , storeTo(grOpts_.defines, parseConst)->composing()->arg("<id>=<term>"), "Replace term occurences of <id> with <term>")
+        ("const,c"                  , storeTo(grOpts_.defines, parseConst)->composing()->arg("<id>=<term>"), "Replace term occurrences of <id> with <term>")
         ("lparse-rewrite"           , flag(grOpts_.lpRewrite = false), "Use together with --text to inspect lparse rewriting")
         ("lparse-debug"             , storeTo(grOpts_.lparseDebug = Gringo::Output::LparseDebug::NONE, values<Gringo::Output::LparseDebug>()
           ("none"  , Gringo::Output::LparseDebug::NONE)
@@ -69,6 +68,7 @@ void ClingoApp::initOptions(ProgramOptions::OptionContext& root) {
          "      [no-]variable-unbounded:    $x > 10.\n"
          "      [no-]global-variable:       :- #count { X } = 1, X = 1.\n")
         ("rewrite-minimize"         , flag(grOpts_.rewriteMinimize = false), "Rewrite minimize constraints into rules")
+        ("keep-facts"               , flag(grOpts_.keepFacts = false), "Do not remove facts from normal rules")
         ("foobar,@4"                , storeTo(grOpts_.foobar, parseFoobar) , "Foobar")
         ;
     root.add(gringo);
@@ -158,8 +158,7 @@ void ClingoApp::run(Clasp::ClaspFacade& clasp) {
         ProgramBuilder* prg = &clasp.start(claspConfig_, pt);
         grOpts_.verbose = verbose() == UINT_MAX;
         Asp::LogicProgram* lp = mode_ != mode_gringo ? static_cast<Asp::LogicProgram*>(prg) : 0;
-        DefaultGringoModule module;
-        grd = Gringo::gringo_make_unique<ClingoControl>(module, mode_ == mode_clingo, clasp_.get(), claspConfig_, std::bind(&ClingoApp::handlePostGroundOptions, this, _1), std::bind(&ClingoApp::handlePreSolveOptions, this, _1));
+        grd = Gringo::gringo_make_unique<ClingoControl>(module.scripts, mode_ == mode_clingo, clasp_.get(), claspConfig_, std::bind(&ClingoApp::handlePostGroundOptions, this, _1), std::bind(&ClingoApp::handlePreSolveOptions, this, _1));
         grd->parse(claspAppOpts_.input, grOpts_, lp);
         grd->main();
     }
