@@ -53,6 +53,7 @@ struct GringoOptions {
     bool                        wNoVariableUnbounded  = false;
     bool                        wNoGlobalVariable     = false;
     bool                        rewriteMinimize       = false;
+    bool                        keepFacts             = false;
     Foobar foobar;
 };
 
@@ -89,6 +90,7 @@ struct IncrementalControl : Gringo::Control, Gringo::GringoModule {
         , parser(pb)
         , opts(opts) {
         using namespace Gringo;
+        out.keepFacts = opts.keepFacts;
         if (opts.wNoOperationUndefined) { message_printer()->disable(W_OPERATION_UNDEFINED); }
         if (opts.wNoAtomUndef)          { message_printer()->disable(W_ATOM_UNDEFINED); }
         if (opts.wNoFileIncluded)       { message_printer()->disable(W_FILE_INCLUDED); }
@@ -241,7 +243,7 @@ protected:
         OptionGroup gringo("Gringo Options");
         gringo.addOptions()
             ("text,t"                   , flag(grOpts_.text = false)     , "Print plain text format")
-            ("const,c"                  , storeTo(grOpts_.defines, parseConst)->composing()->arg("<id>=<term>"), "Replace term occurences of <id> with <term>")
+            ("const,c"                  , storeTo(grOpts_.defines, parseConst)->composing()->arg("<id>=<term>"), "Replace term occurrences of <id> with <term>")
             ("lparse-rewrite"           , flag(grOpts_.lpRewrite = false), "Use together with --text to inspect lparse rewriting")
             ("lparse-debug"             , storeTo(grOpts_.lparseDebug = Gringo::Output::LparseDebug::NONE, values<Gringo::Output::LparseDebug>()
               ("none"  , Gringo::Output::LparseDebug::NONE)
@@ -259,6 +261,7 @@ protected:
              "      [no-]variable-unbounded:    $x > 10.\n"
              "      [no-]global-variable:       :- #count { X } = 1, X = 1.\n")
             ("rewrite-minimize"         , flag(grOpts_.rewriteMinimize = false), "Rewrite minimize constraints into rules")
+            ("keep-facts"               , flag(grOpts_.keepFacts = false), "Do not remove facts from normal rules")
             ("foobar,@4"                , storeTo(grOpts_.foobar, parseFoobar), "Foobar")
             ;
         root.add(gringo);
