@@ -20,7 +20,7 @@
 
 #include <lua.hpp>
 #include "gringo/lua.hh"
-#include "clingocontrol.hh"
+#include "clingo/clingocontrol.hh"
 
 namespace {
 
