@@ -54,6 +54,14 @@ inline void shrinkVecTo(T& t, typename T::size_type j) {
 	t.erase(t.begin()+j, t.end());
 }
 
+template <class T>
+inline void growVecTo(T& vec, typename T::size_type j, const typename T::value_type& val = typename T::value_type()) {
+	if (vec.size() < j) {
+		if (vec.capacity() < j) { vec.reserve(j + j / 2); }
+		vec.resize(j, val);
+	}
+}
+
 template <class T>
 void moveDown(T& t, typename T::size_type from, typename T::size_type to) {
 	for (typename T::size_type end = t.size(); from != end;) {
