@@ -70,11 +70,11 @@ inline bool matchStatPath(const char*& x, const char* path) {
  */
 struct CoreStats {
 #define CLASP_CORE_STATS(STAT, SELF, OTHER)     \
-	STAT(uint64 choices;    , STRING(choices)    , SELF.choices    , SELF.choices    += OTHER.choices   )\
-	STAT(uint64 conflicts;  , STRING(conflicts)  , SELF.conflicts  , SELF.conflicts  += OTHER.conflicts )\
-	STAT(uint64 analyzed;   , STRING(analyzed)   , SELF.analyzed   , SELF.analyzed   += OTHER.analyzed  )\
-	STAT(uint64 restarts;   , STRING(restarts)   , SELF.restarts   , SELF.restarts   += OTHER.restarts  )\
-	STAT(uint64 lastRestart;, STRING(lastRestart), SELF.lastRestart, SELF.lastRestart = std::max(SELF.lastRestart, OTHER.lastRestart))
+	STAT(uint64 choices;    , "choices"           , SELF.choices    , SELF.choices    += OTHER.choices   )\
+	STAT(uint64 conflicts;  , "conflicts"         , SELF.conflicts  , SELF.conflicts  += OTHER.conflicts )\
+	STAT(uint64 analyzed;   , "conflicts_analyzed", SELF.analyzed   , SELF.analyzed   += OTHER.analyzed  )\
+	STAT(uint64 restarts;   , "restarts"          , SELF.restarts   , SELF.restarts   += OTHER.restarts  )\
+	STAT(uint64 lastRestart;, "restarts_last"     , SELF.lastRestart, SELF.lastRestart = std::max(SELF.lastRestart, OTHER.lastRestart))
 
 	CoreStats() { reset(); }
 	void reset() { std::memset(this, 0, sizeof(CoreStats)); }
@@ -100,31 +100,31 @@ struct ExtendedStats {
 	typedef ConstraintType type_t;
 	typedef uint64 Array[Constraint_t::max_value];
 #define CLASP_EXTENDED_STATS(STAT, SELF, OTHER)     \
-	STAT(uint64 domChoices; /**< "domain" choices   */, STRING(domChoices) , SELF.domChoices , SELF.domChoices += OTHER.domChoices) \
-	STAT(uint64 models;     /**< number of models   */, STRING(models)     , SELF.models     , SELF.models     += OTHER.models)     \
-	STAT(uint64 modelLits;  /**< DLs in models      */, STRING(modelLits)  , SELF.modelLits  , SELF.modelLits  += OTHER.modelLits)  \
-	STAT(uint64 hccTests;   /**< stability tests    */, STRING(hccTests)   , SELF.hccTests   , SELF.hccTests   += OTHER.hccTests)   \
-	STAT(uint64 hccPartial; /**< partial stab. tests*/, STRING(hccPartial) , SELF.hccPartial , SELF.hccPartial += OTHER.hccPartial) \
-	STAT(uint64 deleted;    /**< lemmas deleted     */, STRING(deleted)    , SELF.deleted    , SELF.deleted    += OTHER.deleted)    \
-	STAT(uint64 distributed;/**< lemmas distributed */, STRING(distributed), SELF.distributed, SELF.distributed+= OTHER.distributed)\
-	STAT(uint64 sumDistLbd; /**< sum of lemma lbds  */, STRING(sumDistLbd) , SELF.sumDistLbd , SELF.sumDistLbd += OTHER.sumDistLbd) \
-	STAT(uint64 integrated; /**< lemmas integrated  */, STRING(integrated) , SELF.integrated , SELF.integrated += OTHER.integrated) \
-	STAT(Array learnts;  /**< lemmas of type t-1    */, "lemmas"           , SELF.lemmas()   , NO_ARG)   \
-	STAT(Array lits;     /**< lits of type t-1      */, "learntLits"       , SELF.learntLits(), NO_ARG)  \
-	STAT(uint32 binary;  /**< binary lemmas         */, STRING(binary)     , SELF.binary  , SELF.binary  += OTHER.binary)  \
-	STAT(uint32 ternary; /**< ternary lemmas        */, STRING(ternary)    , SELF.ternary , SELF.ternary += OTHER.ternary) \
-	STAT(double cpuTime; /**< cpu time used         */, STRING(cpuTime)    , SELF.cpuTime , SELF.cpuTime += OTHER.cpuTime) \
-	STAT(uint64 intImps; /**< implications on integrating*/, STRING(intImps)    , SELF.intImps , SELF.intImps+= OTHER.intImps)  \
-	STAT(uint64 intJumps;/**< backjumps on integrating  */ , STRING(intJumps)   , SELF.intJumps, SELF.intJumps+=OTHER.intJumps) \
-	STAT(uint64 gpLits;  /**< lits in received gps  */, STRING(gpLits)     , SELF.gpLits  , SELF.gpLits  += OTHER.gpLits)  \
-	STAT(uint32 gps;     /**< guiding paths received*/, STRING(gps)        , SELF.gps     , SELF.gps     += OTHER.gps)     \
-	STAT(uint32 splits;  /**< split requests handled*/, STRING(splits)     , SELF.splits  , SELF.splits  += OTHER.splits)  \
-	STAT(NO_ARG       , "conflictLemmas", SELF.lemmas(Constraint_t::learnt_conflict), SELF.learnts[0] += OTHER.learnts[0]) \
-	STAT(NO_ARG       , "loopLemmas"    , SELF.lemmas(Constraint_t::learnt_loop)    , SELF.learnts[1] += OTHER.learnts[1]) \
-	STAT(NO_ARG       , "otherLemmas"   , SELF.lemmas(Constraint_t::learnt_other)   , SELF.learnts[2] += OTHER.learnts[2]) \
-	STAT(NO_ARG       , "conflictLits"  , SELF.lits[Constraint_t::learnt_conflict-1], SELF.lits[0]    += OTHER.lits[0])    \
-	STAT(NO_ARG       , "loopLits"      , SELF.lits[Constraint_t::learnt_loop-1]    , SELF.lits[1]    += OTHER.lits[1])    \
-	STAT(NO_ARG       , "otherLits"     , SELF.lits[Constraint_t::learnt_other-1]   , SELF.lits[2]    += OTHER.lits[2])
+	STAT(uint64 domChoices; /**< "domain" choices   */, "domain_choices"        , SELF.domChoices , SELF.domChoices += OTHER.domChoices) \
+	STAT(uint64 models;     /**< number of models   */, "models"                , SELF.models     , SELF.models     += OTHER.models)     \
+	STAT(uint64 modelLits;  /**< DLs in models      */, "models_level"          , SELF.modelLits  , SELF.modelLits  += OTHER.modelLits)  \
+	STAT(uint64 hccTests;   /**< stability tests    */, "hcc_tests"             , SELF.hccTests   , SELF.hccTests   += OTHER.hccTests)   \
+	STAT(uint64 hccPartial; /**< partial stab. tests*/, "hcc_partial"           , SELF.hccPartial , SELF.hccPartial += OTHER.hccPartial) \
+	STAT(uint64 deleted;    /**< lemmas deleted     */, "lemmas_deleted"        , SELF.deleted    , SELF.deleted    += OTHER.deleted)    \
+	STAT(uint64 distributed;/**< lemmas distributed */, "distributed"           , SELF.distributed, SELF.distributed+= OTHER.distributed)\
+	STAT(uint64 sumDistLbd; /**< sum of lemma lbds  */, "distributed_sum_lbd"   , SELF.sumDistLbd , SELF.sumDistLbd += OTHER.sumDistLbd) \
+	STAT(uint64 integrated; /**< lemmas integrated  */, "integrated"            , SELF.integrated , SELF.integrated += OTHER.integrated) \
+	STAT(Array learnts;  /**< lemmas of type t-1    */, "lemmas"                , SELF.lemmas()   , NO_ARG)   \
+	STAT(Array lits;     /**< lits of type t-1      */, "lits_learnt"           , SELF.learntLits(), NO_ARG)  \
+	STAT(uint32 binary;  /**< binary lemmas         */, "lemmas_binary"         , SELF.binary  , SELF.binary  += OTHER.binary)  \
+	STAT(uint32 ternary; /**< ternary lemmas        */, "lemmas_ternary"        , SELF.ternary , SELF.ternary += OTHER.ternary) \
+	STAT(double cpuTime; /**< cpu time used         */, "cpu_time"              , SELF.cpuTime , SELF.cpuTime += OTHER.cpuTime) \
+	STAT(uint64 intImps; /**< implications on integrating*/, "integrated_imps"  , SELF.intImps , SELF.intImps+= OTHER.intImps)  \
+	STAT(uint64 intJumps;/**< backjumps on integrating  */ , "integrated_jumps" , SELF.intJumps, SELF.intJumps+=OTHER.intJumps) \
+	STAT(uint64 gpLits;  /**< lits in received gps  */, "guiding_paths_lits"    , SELF.gpLits  , SELF.gpLits  += OTHER.gpLits)  \
+	STAT(uint32 gps;     /**< guiding paths received*/, "guiding_paths"         , SELF.gps     , SELF.gps     += OTHER.gps)     \
+	STAT(uint32 splits;  /**< split requests handled*/, "splits"                , SELF.splits  , SELF.splits  += OTHER.splits)  \
+	STAT(NO_ARG       , "lemmas_conflict", SELF.lemmas(Constraint_t::learnt_conflict), SELF.learnts[0] += OTHER.learnts[0]) \
+	STAT(NO_ARG       , "lemmas_loop"    , SELF.lemmas(Constraint_t::learnt_loop)    , SELF.learnts[1] += OTHER.learnts[1]) \
+	STAT(NO_ARG       , "lemmas_other"   , SELF.lemmas(Constraint_t::learnt_other)   , SELF.learnts[2] += OTHER.learnts[2]) \
+	STAT(NO_ARG       , "lits_conflict"  , SELF.lits[Constraint_t::learnt_conflict-1], SELF.lits[0]    += OTHER.lits[0])    \
+	STAT(NO_ARG       , "lits_loop"      , SELF.lits[Constraint_t::learnt_loop-1]    , SELF.lits[1]    += OTHER.lits[1])    \
+	STAT(NO_ARG       , "lits_other"     , SELF.lits[Constraint_t::learnt_other-1]   , SELF.lits[2]    += OTHER.lits[2])
 	
 	ExtendedStats() { reset(); }
 	void reset() { std::memset(this, 0, sizeof(ExtendedStats)); }
@@ -162,13 +162,13 @@ struct ExtendedStats {
 //! A struct for holding (optional) jump statistics.
 struct JumpStats {
 #define CLASP_JUMP_STATS(STAT, SELF, OTHER)     \
-	STAT(uint64 jumps;    , STRING(jumps)    , SELF.jumps    , SELF.jumps    += OTHER.jumps)   \
-	STAT(uint64 bounded;  , STRING(bounded)  , SELF.bounded  , SELF.bounded  += OTHER.bounded)  \
-	STAT(uint64 jumpSum;  , STRING(jumpSum)  , SELF.jumpSum  , SELF.jumpSum  += OTHER.jumpSum)  \
-	STAT(uint64 boundSum; , STRING(boundSum) , SELF.boundSum , SELF.boundSum += OTHER.boundSum) \
-	STAT(uint32 maxJump;  , STRING(maxJump)  , SELF.maxJump  , MAX_MEM(SELF.maxJump, OTHER.maxJump))   \
-	STAT(uint32 maxJumpEx;, STRING(maxJumpEx), SELF.maxJumpEx, MAX_MEM(SELF.maxJumpEx,OTHER.maxJumpEx)) \
-	STAT(uint32 maxBound; , STRING(maxBound) , SELF.maxBound , MAX_MEM(SELF.maxBound, OTHER.maxBound)) 
+	STAT(uint64 jumps;    , "jumps"    , SELF.jumps    , SELF.jumps    += OTHER.jumps)   \
+	STAT(uint64 bounded;  , "jumps_bounded"  , SELF.bounded  , SELF.bounded  += OTHER.bounded)  \
+	STAT(uint64 jumpSum;  , "levels", SELF.jumpSum  , SELF.jumpSum  += OTHER.jumpSum)  \
+	STAT(uint64 boundSum; , "levels_bounded" , SELF.boundSum , SELF.boundSum += OTHER.boundSum) \
+	STAT(uint32 maxJump;  , "max"  , SELF.maxJump  , MAX_MEM(SELF.maxJump, OTHER.maxJump))   \
+	STAT(uint32 maxJumpEx;, "max_executed", SELF.maxJumpEx, MAX_MEM(SELF.maxJumpEx,OTHER.maxJumpEx)) \
+	STAT(uint32 maxBound; , "max_bounded", SELF.maxBound , MAX_MEM(SELF.maxBound, OTHER.maxBound)) 
 
 	JumpStats() { reset(); }
 	void reset(){ std::memset(this, 0, sizeof(*this)); }
@@ -412,15 +412,16 @@ public:
 	//! Propagates the head and calls updateWatch() if necessary.
 	PropResult propagate(Solver& s, Literal, uint32& data);
 	//! Type of clause.
-	ConstraintType type() const    { return ConstraintType(info_.data.type); }
+	ConstraintType type() const       { return ConstraintType(info_.data.type); }
 	//! True if this clause currently is the antecedent of an assignment.
 	bool     locked(const Solver& s) const;
 	//! Returns the activity of this clause.
-	Activity activity() const       { return Activity(info_.data.act, info_.data.lbd); }
+	Activity activity() const         { return Activity(info_.data.act, info_.data.lbd); }
 	//! Halves the activity of this clause.
-	void     decreaseActivity()     { info_.data.act >>= 1; }
+	void     decreaseActivity()       { info_.data.act >>= 1; }
+	void     resetActivity(Activity a){ info_.data.act = std::min(a.activity(),uint32(MAX_ACTIVITY)); info_.data.lbd = std::min(a.lbd(), uint32(MAX_LBD)); }
 	//! Downcast from LearntConstraint.
-	ClauseHead* clause()           { return this; }
+	ClauseHead* clause()              { return this; }
 	
 	// clause interface
 	typedef std::pair<bool, bool> BoolPair;
@@ -779,6 +780,7 @@ public:
 	void   copyAssignment(Assignment& o) const { o.assign_ = assign_; }
 	bool   markUnits()                 { while (units_ != front) { setSeen(trail[units_++].var(), 3u); }  return true; }
 	void   setUnits(uint32 ts)         { units_ = ts; }
+	void   resetPrefs()                { pref_.assign(pref_.size(), ValueSet()); }
 	//@}
 private:
 	static const uint32 elim_mask = uint32(0xFFFFFFF0u);
