@@ -35,15 +35,16 @@
 #endif
 #ifdef _MSC_VER
 #include <float.h>
-#define CLASP_ISNAN(x) (_isnan(x) != 0) 
-#elif defined(__cplusplus) && __cplusplus >= 201103L 
-#include <cmath> 
-#define CLASP_ISNAN(x) std::isnan(x) 
-#else 
-#include <math.h> 
-#define CLASP_ISNAN(x) isnan(x) 
-#endif 
-inline bool isNan(double d) { return CLASP_ISNAN(d); } 
+#define CLASP_ISNAN(x) (_isnan(x) != 0)
+#pragma warning (disable : 4996)
+#elif defined(__cplusplus) && __cplusplus >= 201103L
+#include <cmath>
+#define CLASP_ISNAN(x) std::isnan(x)
+#else
+#include <math.h>
+#define CLASP_ISNAN(x) isnan(x)
+#endif
+inline bool isNan(double d) { return CLASP_ISNAN(d); }
 #undef CLASP_ISNAN
 namespace Clasp { namespace Cli {
 /////////////////////////////////////////////////////////////////////////////////////////
