@@ -110,7 +110,7 @@ Var ClaspBerkmin::getTopMoms(const Solver& s) {
 	Var var   = front_;
 	uint32 ms = momsScore(s, var);
 	uint32 ls = 0;
-	for (Var v = var+1; v <= s.numVars(); ++v) {
+	for (Var v = var+1; v <= s.numProblemVars(); ++v) {
 		if (s.value(v) == value_free && (ls = momsScore(s, v)) > ms) {
 			var = v;
 			ms  = ls;
@@ -635,7 +635,9 @@ const std::size_t domLen_s = std::strlen(domSym_s);
 bool DomEntry::isDomEntry(const SymbolType& sym) { return !sym.name.empty() && std::strncmp(sym.name.c_str(), domSym_s, domLen_s) == 0; }
 bool DomEntry::isHeadOf(const char* head, const SymbolType& domSym) {
 	assert(isDomEntry(domSym));
-	return std::strncmp(head, domSym.name.c_str() + domLen_s, std::strlen(head)) == 0; 
+	std::size_t len = std::strlen(head);
+	const char* dom = domSym.name.c_str() + domLen_s;
+	return std::strncmp(head, dom, len) == 0 && dom[len] == ',';
 }
 bool DomEntry::Cmp::operator()(const SymbolType& lhs, const SymbolType& rhs) const {
 	assert(DomEntry::isDomEntry(lhs) && DomEntry::isDomEntry(rhs));
@@ -649,36 +651,33 @@ bool DomEntry::Lookup::operator()(const SymbolType& domSym, const char* head) co
 	assert(DomEntry::isDomEntry(domSym));
 	return std::strncmp(domSym.name.c_str() + domLen_s, head, std::strlen(head)) < 0;
 }
-void DomEntry::init(Literal lit, const SymbolType& domSym) {
+void DomEntry::init(const SymbolType& atomSym, const SymbolType& domSym) {
 	CLASP_ASSERT_CONTRACT(isDomEntry(domSym));
 	std::memset(this, 0, sizeof(*this));
-	this->lit = lit;
+	this->lit = atomSym.lit;
 	this->cond= domSym.lit;
 	const char* rule = domSym.name.c_str() + domLen_s;
 	// skip head
-	for (int p = 0; *rule && (*rule != ',' || p); ++rule) {
-		if      (*rule == '('){ ++p; }
-		else if (*rule == ')'){ --p; }
-	}
+	rule += std::strlen(atomSym.name.c_str());
 	CLASP_FAIL_IF(!*rule++, "Invalid atom name in heuristic predicate!");
 	// extract modifier
 	if      (std::strncmp(rule, "sign"  , 4) == 0) { this->mod = mod_sign;   rule += 4; }
-	else if (std::strncmp(rule, "true"  , 4) == 0) { this->mod = mod_tf;     rule += 4; this->pref = value_true; }
+	else if (std::strncmp(rule, "true"  , 4) == 0) { this->mod = mod_tf;     rule += 4; this->pref = trueValue(lit); }
 	else if (std::strncmp(rule, "init"  , 4) == 0) { this->mod = mod_init;   rule += 4; }
 	else if (std::strncmp(rule, "level" , 5) == 0) { this->mod = mod_level;  rule += 5; }
-	else if (std::strncmp(rule, "false" , 5) == 0) { this->mod = mod_tf;     rule += 5; this->pref = value_false; }
+	else if (std::strncmp(rule, "false" , 5) == 0) { this->mod = mod_tf;     rule += 5; this->pref = falseValue(lit); }
 	else if (std::strncmp(rule, "factor", 6) == 0) { this->mod = mod_factor; rule += 6; }
 	CLASP_FAIL_IF(*rule++ != ',', "Invalid modifier in heuristic predicate!");
 	// extract value
 	char* next;
 	this->value = Range<int>(INT16_MIN, INT16_MAX).clamp(std::strtol(rule, &next, 10));
 	CLASP_FAIL_IF(rule == next || *(rule = next) == 0, "Invalid value in heuristic predicate!");
+	this->prio = value > 0 ? value : -value;
 	if (mod == mod_sign) {
 		pref  = value_free + ValueRep(value != 0) + ValueRep(value < 0);
 		if (lit.sign() && pref) { pref ^= 3u; }
 		value = pref;
 	}
-	prio = value > 0 ? value : -value;
 	// extract optional priority
 	if (*rule == ',') { // _heuristic/4
 		prio = Range<int>(0, INT16_MAX).clamp(std::strtol(++rule, &next, 10));
@@ -690,7 +689,6 @@ DomainHeuristic::DomainHeuristic(double d, const HeuParams& params) : ClaspVsids
 	frames_.push_back(Frame(0,UINT32_MAX));
 }
 DomainHeuristic::~DomainHeuristic() {
-	delete domMin_;
 }
 void DomainHeuristic::setDefaultMod(GlobalModifier mod, uint32 prefSet) {
 	defMod_ = (uint16)mod;
@@ -731,46 +729,60 @@ void DomainHeuristic::initScores(Solver& s, bool moms) {
 	std::sort(domTab.begin(), domTab.end(), DomEntry::Cmp());
 	// apply modifications to relevant atoms
 	Literal dyn;
-	uint32 nPrio  = (uint32)prios_.size(), cPrio = nPrio, inc = uint32(s.symbolTable().incremental());
+	uint32 nKey   = (uint32)prios_.size(), uKey = nKey;
 	uint32 nRules = (uint32)domTab.size();
 	if (&s == s.sharedContext()->master()) { domMin_ = &s.sharedContext()->symbolTable().domLits; }
+	typedef PodVector<std::pair<Var, uint32> >::type InitVec;
+	InitVec initVals;
 	for (SymbolTable::const_iterator sIt = s.symbolTable().begin(), end = s.symbolTable().end(); sIt != end && nRules; ++sIt) {
-		if (sIt->second.name.empty() || DomEntry::isDomEntry(sIt->second) || s.topValue(sIt->second.lit.var()) != value_free) { continue; }
+		if (sIt->second.name.empty() || s.topValue(sIt->second.lit.var()) != value_free) { continue; }
 		const char* head = sIt->second.name.c_str();
 		Var         ev   = 0;
 		int16       init = 0;
 		for (DomTable::const_iterator rIt = std::lower_bound(domTab.begin(), domTab.end(), head, DomEntry::Lookup()); rIt != domTab.end() && DomEntry::isHeadOf(head, *rIt); ++rIt, --nRules) {
 			DomEntry e;
-			e.init(sIt->second.lit, *rIt);
+			e.init(sIt->second, *rIt);
 			if (e.mod == DomEntry::mod_init && !s.isTrue(e.cond))   { continue; }
 			ev = e.lit.var();
-			if (score_[ev].domKey >= nPrio){
-				score_[ev].setDom(nPrio++);
+			if (score_[ev].domKey >= nKey){
+				score_[ev].setDom(nKey++);
 				prios_.push_back(DomPrio());
 				prios_.back().clear();
-				if (inc) { cPrio = nPrio; }
 			}
 			int mods = 1;
 			if (e.mod == DomEntry::mod_tf) { e.mod = DomEntry::mod_level; ++mods; }
 			for (;;) {
 				if (e.prio >= prio(ev, e.mod)){
 					if      (s.isTrue(e.cond)) { prio(ev, e.mod) = e.prio; }
-					else if (e.cond != dyn)    { s.addWatch(e.cond, this, (uint32)actions_.size()); dyn = e.cond; inc = true; }
+					else if (e.cond != dyn)    { s.addWatch(dyn = e.cond, this, (uint32)actions_.size()); }
 					else                       { actions_.back().comp = 1; }
-					if (addAction(s, e, init)) { cPrio = nPrio; }
+					if (addAction(s, e, init) && score_[ev].domKey >= uKey) {
+						uKey = score_[ev].domKey + 1;
+					}
 				}
 				if (--mods == 0) { break; }
 				e.mod   = DomEntry::mod_sign;
 				e.value = (int16)e.pref;
 			}
 		}
-		if (ev) {
-			if (init)          { score_[ev].value += init;  }
-			if (nPrio != cPrio){ --nPrio; prios_.pop_back();}
+		if (ev && init) {
+			initVals.push_back(std::make_pair(ev, uint32(init)));
 		}
-		assert(nPrio == cPrio);
 	}
+	std::sort(initVals.begin(), initVals.end());
+	for (InitVec::const_iterator it = initVals.begin(), end = initVals.end(); it != end; ++it) {
+		uint32 val = it->second;
+		for (InitVec::const_iterator j = it + 1; j != end && j->first == it->first; it = j++) {
+			if (j->second > val) { val = j->second; }
+		}
+		score_[it->first].value += val;
+	}
+	InitVec().swap(initVals);
 	DomTable().swap(domTab);
+	if (s.symbolTable().incremental()) { uKey = nKey; }
+	if (uKey != nKey) {
+		PrioVec(prios_.begin(), prios_.begin()+uKey).swap(prios_);
+	}
 	// apply default modification
 	if (defMod_) {
 		MinimizeConstraint* m = s.enumerationConstraint() ? static_cast<EnumerationConstraint*>(s.enumerationConstraint())->minimizer() : 0;
