@@ -18,10 +18,8 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 #ifdef _MSC_VER
-#pragma warning (disable : 4146) // unary minus operator applied to unsigned type, result still unsigned
 #pragma warning (disable : 4996) // 'std::_Fill_n' was declared deprecated
 #endif
-
 #include <clasp/logic_program_types.h>
 #include <clasp/logic_program.h>
 #include <clasp/solver.h>
@@ -498,7 +496,7 @@ bool SccChecker::onNode(PrgNode* n, NodeType t, Call& c, uint32 data) {
 }
 /////////////////////////////////////////////////////////////////////////////////////////
 PrgNode::PrgNode(uint32 id, bool checkScc) 
-	: litIdx_(noIdx), noScc_(1-checkScc), id_(id), val_(value_free), eq_(0), seen_(0) {
+	: litIdx_(noIdx), noScc_(uint32(!checkScc)), id_(id), val_(value_free), eq_(0), seen_(0) {
 	static_assert(sizeof(PrgNode) == sizeof(uint64), "Unsupported Alignment");
 }
 /////////////////////////////////////////////////////////////////////////////////////////
@@ -506,7 +504,7 @@ PrgNode::PrgNode(uint32 id, bool checkScc)
 /////////////////////////////////////////////////////////////////////////////////////////
 PrgHead::PrgHead(uint32 id, NodeType t, uint32 data, bool checkScc)
 	: PrgNode(id, checkScc)
-	, data_(data), upper_(0), dirty_(0), state_(0), isAtom_(t == PrgEdge::ATOM_NODE)  {
+	, data_(data), upper_(0), dirty_(0), freeze_(0), isAtom_(t == PrgEdge::ATOM_NODE)  {
 	struct X { uint64 x; EdgeVec y; uint32 z; };
 	static_assert(sizeof(PrgHead) == sizeof(X), "Unsupported Alignment");
 }
@@ -544,7 +542,7 @@ bool PrgHead::simplifySupports(LogicProgram& prg, bool strong, uint32* numDiffSu
 		for (it = supports_.begin(); it != supports_.end(); ++it) {
 			PrgNode* x = prg.getSupp(*it);
 			if (x->relevant() && x->value() != value_false && (!strong || x->hasVar())) {
-				if      (!x->seen()) { *j++ = *it; x->markSeen(true); }
+				if      (!x->seen()) { *j++ = *it; x->setSeen(true); }
 				else if (!choices)   { continue; }
 				else                 {
 					for (n = supports_.begin(); n != it && n->node() != it->node(); ++n) {;}
@@ -563,7 +561,7 @@ bool PrgHead::simplifySupports(LogicProgram& prg, bool strong, uint32* numDiffSu
 		choices    = 0;
 		for (it = supports_.begin(); it != supports_.end(); ++it) {
 			PrgNode* x = prg.getSupp(*it);
-			x->markSeen(false);
+			x->setSeen(false);
 			if (strong && ctx.marked(x->literal())) { ctx.unmark(x->var()); }
 			if (it->isChoice()) {
 				++choices;
@@ -704,8 +702,9 @@ bool PrgAtom::propagateValue(LogicProgram& prg, bool backprop) {
 	ValueRep val = value();
 	assert(val != value_free);
 	// propagate value forward
+	Literal dep = posLit(id());
 	for (dep_iterator it = deps_.begin(), end = deps_end(); it != end; ++it) {
-		if (!prg.getBody(it->var())->propagateAssigned(prg, *it, val)) {
+		if (!prg.getBody(it->var())->propagateAssigned(prg, dep ^ it->sign(), val)) {
 			return false;
 		}
 	}
@@ -873,6 +872,13 @@ void PrgBody::removeHead(PrgHead* h, EdgeType t) {
 	}
 }
 
+bool PrgBody::hasHead(PrgHead* h, EdgeType t) const {
+	if (!hasHeads()) { return false;  }
+	PrgEdge x = PrgEdge::newEdge(h->id(), t, h->isAtom() ? PrgEdge::ATOM_NODE : PrgEdge::DISJ_NODE);
+	head_iterator it = sHead_ != 0 || !extHead() ? std::find(heads_begin(), heads_end(), x) : std::lower_bound(heads_begin(), heads_end(), x);
+	return it != heads_end() && *it == x;
+}
+
 bool PrgBody::eraseHead(PrgEdge h) {
 	PrgEdge* it = const_cast<PrgEdge*>(std::find(heads_begin(), heads_end(), h));
 	if (it != heads_end()) {
@@ -906,17 +912,17 @@ bool PrgBody::simplifyBody(LogicProgram& prg, bool strong, uint32* eqId) {
 	for (Literal* it = j, *end = j + size(); it != end; ++it) {
 		a       = it->var();
 		isEq    = a != prg.getEqAtom(a);
-		oldHash+= hashId(it->sign()?-a:a);
+		oldHash+= hashLit(*it);
 		if (isEq) {
 			prg.getAtom(a)->removeDep(id(), !it->sign()); // remove old edge
-			*it   = prg.getAtom(a)->eqGoal(it->sign());   // replace with eq goal
-			a     = it->var();                            // and check it
+			*it = prg.getAtom(a)->eqGoal(it->sign());     // replace with eq goal
+			a   = it->var();                              // and check it
 		}
 		Literal aLit = it->sign() ? ~prg.getAtom(a)->literal() : prg.getAtom(a)->literal();
 		ValueRep v   = prg.getAtom(a)->value();
 		mark         = strong || prg.getAtom(a)->hasVar();
 		if (strong && !prg.getAtom(a)->hasVar()) {
-			v          = value_false;
+			v = value_false;
 		}
 		if (v == value_weak_true && it->sign()) {
 			v = value_true;
@@ -924,7 +930,7 @@ bool PrgBody::simplifyBody(LogicProgram& prg, bool strong, uint32* eqId) {
 		if (v == value_true || v == value_false) { // truth value is known - remove subgoal
 			if (v == trueValue(*it)) {
 				// subgoal is true: decrease necessary lower bound
-				bound -= weight(it - lits);
+				bound -= weight(uint32(it - lits));
 			}
 			prg.getAtom(a)->removeDep(id(), !it->sign());
 		}
@@ -934,14 +940,12 @@ bool PrgBody::simplifyBody(LogicProgram& prg, bool strong, uint32* eqId) {
 				todo.addToBody(Literal(*it));
 				++todos;
 			}
-			*j++    = *it;       // copy literal
-			if (hasWeights()) {  // and weight
-				*jw++ = weight(it - lits);
-			}
+			*j++ = *it;  // copy literal and optionally weight
+			if (jw) { *jw++ = weight(uint32(it - lits)); }
 		}
 		else { // body contains aLit more than once 
 			if (type() != BodyInfo::NORMAL_BODY) { // merge subgoal
-				if (!hasWeights()) {
+				if (!jw) {
 					SumExtra* extra = SumExtra::create(size());
 					extra->bound    = this->bound();
 					extra->sumW     = this->sumW();
@@ -951,8 +955,8 @@ bool PrgBody::simplifyBody(LogicProgram& prg, bool strong, uint32* eqId) {
 					data_.ext[0]    = extra;
 					jw              = extra->weights + (it - lits);
 				}
-				else { w          = weight(it - lits); }
-				uint32 pos        = findLit(prg, aLit);
+				else { w = weight(uint32(it - lits)); }
+				uint32 pos = findLit(prg, aLit);
 				data_.ext[0]->weights[pos] += w;
 			}
 			else { // ignore if normal
@@ -967,19 +971,19 @@ bool PrgBody::simplifyBody(LogicProgram& prg, bool strong, uint32* eqId) {
 	// unmark atoms, compute new hash value,
 	// and restore pos | neg partition in case
 	// we changed some positive goals to negative ones 
-	size_           = j - lits;
-	if (jw) jw      = data_.ext[0]->weights;
-	uint32 p = 0, n = size_, newHash = 0;
-	weight_t sumW   = 0, reachW = 0;
-	for (uint32 a, h, i; p < n;) {
-		if      (!lits[p].sign())      { h = hashId( (a = lits[i = p++].var()));  }
-		else if (lits[n-1].sign())     { h = hashId(-(a = lits[i = --n].var()));  }
+	size_          = j - lits;
+	if (jw) jw     = data_.ext[0]->weights;
+	uint32 newHash = 0;
+	weight_t sumW  = 0, reachW = 0;
+	for (uint32 p = 0, n = size_, i, h; p < n;) {
+		if      (!lits[p].sign())      { h = hashLit(lits[i = p++]);  }
+		else if (lits[n-1].sign())     { h = hashLit(lits[i = --n]);  }
 		else /* restore pos|neg order */ {
 			std::swap(lits[p], lits[n-1]);
 			if (jw) { std::swap(jw[p], jw[n-1]); }
 			continue;
 		}
-		assert(a == lits[i].var());
+		a = lits[i].var();
 		if (todos && todo.inBody(lits[i])) {
 			prg.getAtom(a)->addDep(id(), !lits[i].sign());
 			todo.clearBody(lits[i]);
@@ -1002,7 +1006,7 @@ bool PrgBody::simplifyBody(LogicProgram& prg, bool strong, uint32* eqId) {
 		ctx.unmark( v );
 		newHash += h;
 	}
-	bool ok = normalize(bound, sumW, reachW, newHash);
+	bool ok = normalize(prg, bound, sumW, reachW, newHash);
 	if (ok) {
 		Var xId = id();
 		if (oldHash != newHash) {
@@ -1014,7 +1018,7 @@ bool PrgBody::simplifyBody(LogicProgram& prg, bool strong, uint32* eqId) {
 	return ok && (value() == value_free || propagateValue(prg, prg.options().backprop));	
 }
 
-bool PrgBody::normalize(weight_t bound, weight_t sumW, weight_t reachW, uint32& hashOut) {
+bool PrgBody::normalize(const LogicProgram& prg, weight_t bound, weight_t sumW, weight_t reachW, uint32& hashOut) {
 	BodyInfo::BodyType nt = (sumW == bound || size() == 1) ? BodyInfo::NORMAL_BODY : type();
 	bool ok = true;
 	if (sumW >= bound && type() != BodyInfo::NORMAL_BODY) {
@@ -1027,6 +1031,9 @@ bool PrgBody::normalize(weight_t bound, weight_t sumW, weight_t reachW, uint32&
 		}
 	}
 	if (bound <= 0) {
+		for (uint32 i = 0, myId = id(); i != size_; ++i) {
+			prg.getAtom(goal(i).var())->removeDep(myId, !goal(i).sign());
+		}
 		size_  = 0; hashOut = 0, unsupp_ = 0;
 		nt     = BodyInfo::NORMAL_BODY;
 		ok     = assignValue(value_true);
@@ -1122,7 +1129,11 @@ bool PrgBody::mergeHeads(LogicProgram& prg, PrgBody& heads, bool strong, bool si
 		else {
 			assert(heads.sHead_ == 0 && "Heads to merge not simplified!");
 			heads.prepareSimplifyHeads(prg, rs);
-			ok = simplifyHeadsImpl(prg, *this, rs, strong) && heads.simplifyHeadsImpl(prg, *this, rs, strong);
+			if (!simplifyHeadsImpl(prg, *this, rs, strong) && !assignValue(value_false)) {
+				rs.clearAll();
+				return false;
+			}
+			ok = heads.simplifyHeadsImpl(prg, *this, rs, strong);
 			assert(ok || heads.heads_begin() == heads.heads_end());
 		}
 		// clear temporary flags & reestablish ordering
@@ -1170,19 +1181,19 @@ bool PrgBody::superfluousHead(const LogicProgram& prg, const PrgHead* head, PrgE
 	else { assert(it.isDisj()); 
 		// check each contained atom
 		const PrgDisj* dis = static_cast<const PrgDisj*>(head);
-		for (PrgDisj::atom_iterator it = dis->begin(), end = dis->end(); it != end; ++it) {
-			if (rs.inBody(posLit(it->node())) || rs.inHead(PrgEdge::newEdge(it->node(), PrgEdge::NORMAL_EDGE, PrgEdge::ATOM_NODE))) {
+		for (PrgDisj::atom_iterator aIt = dis->begin(), aEnd = dis->end(); aIt != aEnd; ++aIt) {
+			if (rs.inBody(posLit(aIt->node())) || rs.inHead(PrgEdge::newEdge(aIt->node(), PrgEdge::NORMAL_EDGE, PrgEdge::ATOM_NODE))) {
 				return true;
 			}
-			if (prg.isFact(prg.getAtom(it->node()))) {
+			if (prg.isFact(prg.getAtom(aIt->node()))) {
 				return true;
 			}
 		}
 		// check for subsumption
-		if (prg.options().iters == UINT32_MAX) {
-			for (head_iterator it = heads_begin(), end = heads_end(); it != end; ++it) {
-				if (it->isDisj() && prg.getDisj(it->node())->size() < dis->size()) {
-					const PrgDisj* other = prg.getDisj(it->node());
+		if (prg.options().iters == LogicProgram::AspOptions::MAX_EQ_ITERS) {
+			for (head_iterator hIt = heads_begin(), hEnd = heads_end(); hIt != hEnd; ++hIt) {
+				if (hIt->isDisj() && prg.getDisj(hIt->node())->size() < dis->size()) {
+					const PrgDisj* other = prg.getDisj(hIt->node());
 					for (PrgDisj::atom_iterator a = other->begin(), aEnd = other->end(); a != aEnd && other; ++a) {
 						if (std::find(dis->begin(), dis->end(), *a) == dis->end()) {
 							other = 0;
@@ -1238,7 +1249,7 @@ bool PrgBody::eqLits(WeightLitVec& vec, bool& sorted) const {
 	if (!sorted && vec.size() <= 10) {
 		for (WeightLitVec::const_iterator it = vec.begin(), end = vec.end(); it != end; ++it) {
 			const Literal* x = std::find(goals_begin(), goals_end(), it->first);
-			if (x == goals_end() || weight(std::distance(goals_begin(), x)) != it->second) {
+			if (x == goals_end() || weight((uint32)std::distance(goals_begin(), x)) != it->second) {
 				return false;
 			}
 		}
@@ -1257,7 +1268,7 @@ bool PrgBody::eqLits(WeightLitVec& vec, bool& sorted) const {
 bool PrgBody::propagateSupported(Var v) {
 	weight_t w = 1;
 	if (hasWeights()) {
-		uint32 pos = std::distance(goals_begin(), std::find(goals_begin(), goals_end(), posLit(v)));
+		uint32 pos = (uint32)std::distance(goals_begin(), std::find(goals_begin(), goals_end(), posLit(v)));
 		w          = weight(pos);
 	}
 	return (unsupp_ -= w) <= 0;
@@ -1265,6 +1276,7 @@ bool PrgBody::propagateSupported(Var v) {
 
 bool PrgBody::propagateAssigned(LogicProgram& prg, Literal p, ValueRep v) {
 	if (!relevant()) return true;
+	assert(std::find(goals_begin(), goals_end(), p) != goals_end());
 	markDirty();
 	ValueRep x = v == value_weak_true ? value_true : v;
 	weight_t w = 1; // TODO: find weight of p for weight rule
@@ -1280,11 +1292,8 @@ bool PrgBody::propagateAssigned(LogicProgram& prg, Literal p, ValueRep v) {
 bool PrgBody::propagateAssigned(LogicProgram& prg, PrgHead* h, EdgeType t) {
 	if (!relevant()) return true;
 	markHeadsDirty();
-	if (h->value() == value_false) {
-		if (t == PrgEdge::NORMAL_EDGE && value() != value_false) {
-			return assignValue(value_false) && propagateValue(prg, prg.options().backprop);
-		}
-		eraseHead(PrgEdge::newEdge(h->id(), t, h->isAtom() ? PrgEdge::ATOM_NODE : PrgEdge::DISJ_NODE));
+	if (h->value() == value_false && eraseHead(PrgEdge::newEdge(h->id(), t, h->isAtom() ? PrgEdge::ATOM_NODE : PrgEdge::DISJ_NODE)) && t == PrgEdge::NORMAL_EDGE) {
+		return value() == value_false || (assignValue(value_false) && propagateValue(prg, prg.options().backprop));
 	}
 	return true;
 }
@@ -1341,7 +1350,6 @@ bool PrgBody::addConstraints(const LogicProgram& prg, ClauseCreator& gc) {
 		Literal negB= ~literal();
 		gc.start().add(literal()); 
 		for (const Literal* it = goals_begin(), *end = goals_end(); it != end; ++it) {
-			assert(it->var() != 0);
 			Literal li = prg.getAtom(it->var())->literal() ^ it->sign();
 			if (li == literal()) { taut = true; continue; }
 			if (!prg.ctx()->addBinary(negB, li)) { // [~B li]
