@@ -136,18 +136,17 @@ struct ParallelSolve::SharedData {
 	typedef tbb::concurrent_queue<const LitVec*> queue;
 	enum MsgFlag {
 		terminate_flag        = 1u, sync_flag  = 2u,  split_flag    = 4u, 
-		restart_flag          = 8u, unsat_flag = 16u, complete_flag = 32u,
-		interrupt_flag        = 64u,  // set on terminate from outside
-		allow_split_flag      = 128u, // set if splitting mode is active
-		forbid_restart_flag   = 256u, // set if restarts are no longer allowed
-		cancel_restart_flag   = 512u, // set if current restart request was cancelled by some thread
-		restart_abandoned_flag=1024u  // set to signal that threads must not give up their gp
+		restart_flag          = 8u, complete_flag = 16u,
+		interrupt_flag        = 32u, // set on terminate from outside
+		allow_split_flag      = 64u, // set if splitting mode is active
+		forbid_restart_flag   = 128u,// set if restarts are no longer allowed
+		cancel_restart_flag   = 256u,// set if current restart request was cancelled by some thread
+		restart_abandoned_flag= 512u,// set to signal that threads must not give up their gp
 	};
 	enum Message {
 		msg_terminate      = (terminate_flag),
 		msg_interrupt      = (terminate_flag | interrupt_flag),
 		msg_sync_restart   = (sync_flag | restart_flag),
-		msg_sync_unsat     = (sync_flag | unsat_flag | restart_flag),
 		msg_split          = split_flag
 	};
 	SharedData() : path(0) { reset(0); control = 0; }
@@ -158,6 +157,7 @@ struct ParallelSolve::SharedData {
 		globalR.reset();
 		maxConflict = globalR.current();
 		error       = 0;
+		initMask    = 0;
 		ctx         = a_ctx;
 		path        = 0;
 		nextId      = 1;
@@ -169,6 +169,16 @@ struct ParallelSolve::SharedData {
 			if (a != path) { delete a; }
 		}
 	}
+	const LitVec* requestWork(uint32 id) {
+		// try to get initial path
+		uint64 m(uint64(1) << id);
+		uint64 init = initMask;
+		if ((m & init) == m) { initMask -= m; return path; }
+		// try to get path from split queue
+		const LitVec* res = 0;
+		if (workQ.try_pop(res)) { return res; }
+		return 0;
+	}
 	// MESSAGES
 	bool        hasMessage()  const { return (control & uint32(7)) != 0; }
 	bool        synchronize() const { return (control & uint32(sync_flag))      != 0; }
@@ -191,6 +201,7 @@ struct ParallelSolve::SharedData {
 	uint64           error;       // bitmask of erroneous solvers
 	SharedContext*   ctx;         // shared context object
 	const LitVec*    path;        // initial guiding path - typically empty
+	atomic<uint64>   initMask;    // bitmask for initialization
 	Timer<RealTime>  syncT;       // thread sync time
 	mutex            modelM;      // model-mutex 
 	BarrierSemaphore workSem;     // work-semaphore
@@ -240,7 +251,7 @@ ParallelSolve::ParallelSolve(Enumerator* e, const ParallelSolveOptions& opts)
 	, intGrace_(1024)
 	, intTopo_(opts.integrate.topo)
 	, intFlags_(ClauseCreator::clause_not_root_sat | ClauseCreator::clause_no_add)
-	, initialGp_(opts.algorithm.mode == ParallelSolveOptions::Algorithm::mode_split ? gp_split : gp_fixed) {
+	, modeSplit_(opts.algorithm.mode == ParallelSolveOptions::Algorithm::mode_split) {
 	setRestarts(opts.restarts.maxR, opts.restarts.sched);
 	setIntegrate(opts.integrate.grace, opts.integrate.filter);
 }
@@ -261,12 +272,24 @@ bool ParallelSolve::beginSolve(SharedContext& ctx) {
 	assert(ctx.concurrency() && "Illegal number of threads");
 	if (shared_->terminate()) { return false; }
 	shared_->reset(&ctx);
-	shared_->setControl(initialGp_ == gp_split ? SharedData::allow_split_flag : SharedData::forbid_restart_flag);
-	shared_->setControl(SharedData::sync_flag); // force initial sync with all threads
-	shared_->modCount = uint32(enumerator().optimize());
-	if (distribution_.types != 0 && ctx.distributor.get() == 0) {
-		ctx.distributor.reset(new mt::GlobalQueue(distribution_, ctx.concurrency(), intTopo_));
+	if (!enumerator().supportsParallel() && numThreads() > 1) {
+		ctx.report(warning(Event::subsystem_solve, "Selected reasoning mode implies #Threads=1."));
+		shared_->workSem.unsafe_init(1);
+		modeSplit_ = false;
+		ctx.setConcurrency(1, SharedContext::mode_reserve);
 	}
+	shared_->setControl(modeSplit_ ? SharedData::allow_split_flag : SharedData::forbid_restart_flag);
+	shared_->modCount = uint32(enumerator().optimize());
+	if (distribution_.types != 0 && ctx.distributor.get() == 0 && numThreads() > 1) {
+		if (distribution_.mode == ParallelSolveOptions::Distribution::mode_local) {
+			ctx.distributor.reset(new mt::LocalDistribution(distribution_, ctx.concurrency(), intTopo_));
+		}
+		else {
+			ctx.distributor.reset(new mt::GlobalDistribution(distribution_, ctx.concurrency(), intTopo_));
+		}
+	}
+	shared_->setControl(SharedData::sync_flag); // force initial sync with all threads
+	shared_->syncT.start();
 	reportProgress(MessageEvent(*ctx.master(), "SYNC", MessageEvent::sent));
 	for (uint32 i = 1; i != ctx.concurrency(); ++i) {
 		uint32 id = shared_->nextId++;
@@ -301,16 +324,15 @@ void ParallelSolve::allocThread(uint32 id, Solver& s, const SolveParams& p) {
 		thread_  = new ParallelHandler*[n];
 		std::fill(thread_, thread_+n, static_cast<ParallelHandler*>(0));
 	}
-	#pragma message TODO("replace with CACHE_LINE_ALIGNED alloc")
-	uint32 b   = ((sizeof(ParallelHandler)+63) / 64) * 64;
-	thread_[id]= new (::operator new( b )) ParallelHandler(*this, s, p);
+	size_t sz   = ((sizeof(ParallelHandler)+63) / 64) * 64;
+	thread_[id] = new (alignedAlloc(sz, 64)) ParallelHandler(*this, s, p);
 }
 
 void ParallelSolve::destroyThread(uint32 id) {
 	if (thread_ && thread_[id]) {
 		assert(!thread_[id]->joinable() && "Shutdown not completed!");
 		thread_[id]->~ParallelHandler();
-		::operator delete(thread_[id]);
+		alignedFree(thread_[id]);
 		thread_[id] = 0;
 		if (id == masterId) {
 			delete [] thread_;
@@ -354,7 +376,6 @@ bool ParallelSolve::doSolve(SharedContext& ctx, const LitVec& path) {
 		assert(s.id() == masterId);
 		allocThread(masterId, s, ctx.configuration()->search(masterId));
 		shared_->path = &path;
-		shared_->syncT.start();
 		solveParallel(masterId);
 		reportProgress(message(Event::subsystem_solve, "Joining with other threads", &s));
 		joinThreads();
@@ -385,18 +406,20 @@ void ParallelSolve::solveParallel(uint32 id) {
 		thread_[id]->attach(*shared_->ctx);
 		BasicSolve solve(s, p, &lim);
 		agg.enableStats(s.stats);
-		for (ValueRep last = value_free; requestWork(s, a); solve.reset()) {
+		for (GpType t; requestWork(s, a); solve.reset()) {
 			agg.accu(s.stats);
 			s.stats.reset();
-			last = thread_[id]->solveGP(solve, *a, a.is_owner() ? gp_split : initialGp_, shared_->maxConflict);
-			if (last == value_free) { terminate(s, false); }
+			thread_[id]->setGpType(t = ((a.is_owner() || modeSplit_) ? gp_split : gp_fixed));
+			if (enumerator().start(s, *a, a.is_owner()) && thread_[id]->solveGP(solve, t, shared_->maxConflict) == value_free) {
+				terminate(s, false);
+			}
 			s.clearStopConflict();
-			enumerator().relax(s);
+			enumerator().end(s);
 		}
 	}
-	catch (const std::bad_alloc&)      { exception(id,a,error_oom, "ERROR: std::bad_alloc" ); }
-	catch (const std::runtime_error& e){ exception(id,a,error_runtime, e.what()); }
-	catch (...)                        { exception(id,a,error_other, "ERROR: unknown");  }
+	catch (const std::bad_alloc&)  { exception(id,a,error_oom, "ERROR: std::bad_alloc" ); }
+	catch (const std::exception& e){ exception(id,a,error_runtime, e.what()); }
+	catch (...)                    { exception(id,a,error_other, "ERROR: unknown");  }
 	assert(shared_->terminate() || thread_[id]->error() != error_none);
 	// this thread is leaving
 	shared_->workSem.removeParty(shared_->terminate());
@@ -419,7 +442,7 @@ void ParallelSolve::exception(uint32 id, PathPtr& path, ErrorCode e, const char*
 			ParallelSolve::doInterrupt();
 			return;
 		}
-		else if (path.get() && thread_[id]->disjointPath()) {
+		else if (path.get() && shared_->allowSplit()) {
 			shared_->workQ.push(path.release());
 			shared_->workSem.up();
 		}
@@ -439,14 +462,19 @@ bool ParallelSolve::doInterrupt() {
 // tries to get new work for the given solver
 bool ParallelSolve::requestWork(Solver& s, PathPtr& out) { 
 	const LitVec* a = 0;
-	while (!shared_->terminate()) {
+	for (int popped = 0; !shared_->terminate();) {
 		// only clear path and stop conflict - we don't propagate() here
 		// because we would then have to handle any eventual conflicts
-		if (!s.popRootLevel(s.rootLevel())) {
+		if (++popped == 1 && !s.popRootLevel(s.rootLevel())) {
 			// s has a real top-level conflict - problem is unsat
 			terminate(s, true);
-		}	
-		else if (a || shared_->workQ.try_pop(a)) {
+		}
+		else if (shared_->synchronize()) {
+			// a synchronize request is active - we are fine with
+			// this but did not yet had a chance to react on it
+			waitOnSync(s);
+		}
+		else if (a || (a = shared_->requestWork(s.id())) != 0) {
 			assert(s.decisionLevel() == 0);
 			// got new work from work-queue
 			out = a;
@@ -457,11 +485,7 @@ bool ParallelSolve::requestWork(Solver& s, PathPtr& out) {
 			// s now has a conflict - either an artifical stop conflict
 			// or a real conflict - we'll handle it in the next iteration
 			// via the call to popRootLevel()
-		}
-		else if (shared_->synchronize()) {
-			// a synchronize request is active - we are fine with
-			// this but did not yet had a chance to react on it
-			waitOnSync(s);
+			popped = 0;
 		}
 		else if (shared_->allowSplit()) {
 			// gp mode is active - request a split	
@@ -487,15 +511,11 @@ bool ParallelSolve::requestWork(Solver& s, PathPtr& out) {
 // check if there is more to do
 void ParallelSolve::terminate(Solver& s, bool complete) {
 	if (!shared_->terminate()) {
-		if (complete && enumerator().optimize() > 1 && s.popRootLevel(s.rootLevel())) {
-			// Problem is unsat and hierarchical optimization is active.
-			// The active level is at its optimum, but lower-priority levels might
-			// still be non-optimal.
-			// Notify other threads to prepare for solving next priority level.
-			assert(s.decisionLevel() == 0);
-			complete = false;
-			reportProgress(MessageEvent(s, "SYNC-UNSAT", MessageEvent::sent));
-			shared_->postMessage(SharedData::msg_sync_unsat, true);
+		if (enumerator().tentative() && complete) {
+			if (shared_->setControl(SharedData::sync_flag|SharedData::complete_flag)) {
+				thread_[s.id()]->setWinner();
+				reportProgress(MessageEvent(s, "SYNC", MessageEvent::sent));
+			}
 		}
 		else {
 			reportProgress(MessageEvent(s, "TERMINATE", MessageEvent::sent));
@@ -512,13 +532,13 @@ bool ParallelSolve::waitOnSync(Solver& s) {
 	if (!thread_[s.id()]->handleRestartMessage()) {
 		shared_->setControl(SharedData::cancel_restart_flag);
 	}
-	bool hasPath = thread_[s.id()]->hasPath();
+	bool hasPath  = thread_[s.id()]->hasPath();
+	bool tentative= enumerator().tentative();
 	if (shared_->workSem.wait()) {
 		// last man standing - complete synchronization request
 		shared_->workReq     = 0;
 		shared_->restartReq  = 0;
-		bool unsat           = shared_->hasControl(SharedData::unsat_flag);
-		bool restart         = !unsat  && shared_->hasControl(SharedData::restart_flag);
+		bool restart         = shared_->hasControl(SharedData::restart_flag);
 		bool init            = true;
 		if (restart) {
 			init = shared_->allowRestart() && !shared_->hasControl(SharedData::cancel_restart_flag);
@@ -527,20 +547,18 @@ bool ParallelSolve::waitOnSync(Solver& s) {
 				? shared_->globalR.current()
 				: UINT64_MAX;
 		}
-		else if (unsat && !enumerator().commitUnsat(s, true)) {
-			// nothing more to do
-			init = false;
-			shared_->setControl(SharedData::terminate_flag | SharedData::complete_flag);
-		}
 		else if (shared_->maxConflict != UINT64_MAX && !shared_->allowRestart()) {
 			shared_->maxConflict = UINT64_MAX;
 		}
-		// add initial path to work queue
 		if (init) { initQueue();  }
 		else      { shared_->setControl(SharedData::restart_abandoned_flag); }
-		shared_->clearControl(SharedData::msg_split | SharedData::msg_sync_unsat | SharedData::msg_sync_restart | SharedData::restart_abandoned_flag | SharedData::cancel_restart_flag);
+		if (tentative && shared_->complete()) {
+			if (enumerator().commitComplete()) { shared_->setControl(SharedData::terminate_flag); }
+			else                               { shared_->modCount = uint32(0); shared_->clearControl(SharedData::complete_flag); }
+		}
+		shared_->clearControl(SharedData::msg_split | SharedData::msg_sync_restart | SharedData::restart_abandoned_flag | SharedData::cancel_restart_flag);
 		shared_->syncT.lap();
-		reportProgress(MessageEvent(s, unsat ? "SYNC-UNSAT" : "SYNC", MessageEvent::completed, shared_->syncT.elapsed()));
+		reportProgress(MessageEvent(s, "SYNC", MessageEvent::completed, shared_->syncT.elapsed()));
 		// wake up all blocked threads
 		shared_->workSem.reset();
 	}
@@ -555,56 +573,75 @@ bool ParallelSolve::waitOnSync(Solver& s) {
 //  heuristic for initial splits?
 void ParallelSolve::initQueue() {
 	shared_->clearQueue();
-	int end = shared_->hasControl(SharedData::allow_split_flag) ? 1 : numThreads();
-	assert(end == 1 || shared_->hasControl(SharedData::forbid_restart_flag));
-	for (int i = 0; i != end; ++i) {
-		shared_->workQ.push(shared_->path);
+	uint64 init = UINT64_MAX;
+	if (shared_->allowSplit()) {
+		if (modeSplit_ && !enumerator().supportsSplitting(*shared_->ctx)) {
+			shared_->ctx->report(warning(Event::subsystem_solve, "Selected strategies imply Mode=compete."));
+			shared_->clearControl(SharedData::allow_split_flag);
+			shared_->setControl(SharedData::forbid_restart_flag);
+			modeSplit_ = false;
+		}
+		else {
+			init = 0;
+			shared_->workQ.push(shared_->path);
+		}
 	}
+	shared_->initMask = init;
+	assert(shared_->allowSplit() || shared_->hasControl(SharedData::forbid_restart_flag));
 }
 
 // adds work to the work-queue
-void ParallelSolve::pushWork(LitVec& work) { 
-	LitVec* v = new LitVec;
-	v->swap(work);
+void ParallelSolve::pushWork(LitVec* v) { 
+	assert(v);
 	shared_->workQ.push(v);
 	shared_->workSem.up();
 }
 
+// called whenever some solver proved unsat
+bool ParallelSolve::commitUnsat(Solver& s) {
+	if (!enumerator().optimize() || shared_->terminate() || shared_->synchronize()) {
+		return false; 
+	}
+	if (!thread_[s.id()]->disjointPath()) {
+		lock_guard<mutex> lock(shared_->modelM);
+		if (!enumerator().commitUnsat(s)) {
+			terminate(s, true);
+			return false;
+		}
+		++shared_->modCount;
+		return true;
+	}
+	return enumerator().commitUnsat(s);
+}
+
 // called whenever some solver has found a model
 bool ParallelSolve::commitModel(Solver& s) { 
-	bool interrupt = false;
-	{
-		// grab lock - models must be processed sequentially
-		// in order to simplify printing and to avoid duplicates
-		// in all non-trivial enumeration modes
-		lock_guard<mutex> lock(shared_->modelM);
-		// first check if the model is still valid once all
-		// information is integrated into the solver
-		bool isModel  = thread_[s.id()]->isModel(s);
-		interrupt     = shared_->terminate();
-		while (!interrupt && (isModel || enumerator().hasModel(s))) {
-			Model m     = enumerator().commitModel(s);
-			if (m.num == 1 && !enumerator().supportsRestarts()) {
-				// switch to backtracking based splitting algorithm
-				// the solver's gp will act as the root for splitting and is
-				// from now on disjoint from all other gps
-				shared_->setControl(SharedData::forbid_restart_flag | SharedData::allow_split_flag);
-				thread_[s.id()]->setGpType(gp_split); 
-				enumerator().setDisjoint(s, true);
-			}
-			if (isModel) { ++shared_->modCount; isModel = false; }
-			if (!reportModel(s, m)) {
-				// must be called while holding the lock - otherwise
-				// we have a race condition with solvers that
-				// are currently blocking on the mutex and we could enumerate 
-				// more models than requested by the user
-				terminate(s, s.decisionLevel() == 0);
-				return false;
-			}
-			interrupt = shared_->interrupt();
+	// grab lock - models must be processed sequentially
+	// in order to simplify printing and to avoid duplicates
+	// in all non-trivial enumeration modes
+	bool stop = false;
+	{lock_guard<mutex> lock(shared_->modelM);
+	// first check if the model is still valid once all
+	// information is integrated into the solver
+	if (thread_[s.id()]->isModel(s) && (stop=shared_->terminate()) == false && enumerator().commitModel(s)) {
+		if (enumerator().lastModel().num == 1 && !enumerator().supportsRestarts()) {
+			// switch to backtracking based splitting algorithm
+			// the solver's gp will act as the root for splitting and is
+			// from now on disjoint from all other gps
+			shared_->setControl(SharedData::forbid_restart_flag | SharedData::allow_split_flag);
+			thread_[s.id()]->setGpType(gp_split); 
+			enumerator().setDisjoint(s, true);
 		}
-	} // end of critical section
-	return !interrupt && (!s.hasConflict() || s.resolveConflict());
+		++shared_->modCount;
+		if ((stop = !reportModel(s)) == true) {
+			// must be called while holding the lock - otherwise
+			// we have a race condition with solvers that
+			// are currently blocking on the mutex and we could enumerate 
+			// more models than requested by the user
+			terminate(s, !moreModels(s));
+		}
+	}}
+	return !stop;
 }
 
 uint64 ParallelSolve::hasErrors() const {
@@ -649,14 +686,14 @@ bool ParallelSolve::handleMessages(Solver& s) {
 		shared_->aboutToSplit();
 		reportProgress(MessageEvent(s, "SPLIT", MessageEvent::received));
 		h->handleSplitMessage();
+		enumerator().setDisjoint(s, true);
 	}
 	return true;
 }
 
 bool ParallelSolve::integrateModels(Solver& s, uint32& upCount) {
 	uint32 gCount = shared_->modCount;
-	return (gCount == upCount || enumerator().hasModel(s))
-	  ||   (enumerator().update(s) && (upCount = gCount) == gCount);
+	return gCount == upCount || (enumerator().update(s) && (upCount = gCount) == gCount);
 }
 
 void ParallelSolve::requestRestart() {
@@ -691,7 +728,6 @@ ParallelHandler::~ParallelHandler() { clearDB(0); delete [] received_; }
 bool ParallelHandler::attach(SharedContext& ctx) {
 	assert(solver_ && params_);
 	gp_.reset();
-	gp_.impl= UINT32_MAX;
 	error_  = 0;
 	win_    = 0;
 	up_     = 0;
@@ -727,20 +763,20 @@ void ParallelHandler::clearDB(Solver* s) {
 	recEnd_= 0;
 }
 
-ValueRep ParallelHandler::solveGP(BasicSolve& solve, const LitVec& gp, GpType t, uint64 restart) {
-	gp_.reset(restart, t);
-	up_  = 1;
-	act_ = 0;
-	const_cast<const ParallelSolve*>(ctrl_)->enumerator()->setDisjoint(solve.solver(), disjointPath());
-	if (!solve.assume(gp)) { return value_false; }
-	act_ = 1; // activate enumerator and bounds
+ValueRep ParallelHandler::solveGP(BasicSolve& solve, GpType t, uint64 restart) {
 	ValueRep res = value_free;
-	for (Solver& s = solve.solver();;) {
+	bool     term= false;
+	Solver&  s   = solve.solver();
+	gp_.reset(restart, t);
+	assert(act_ == 0);
+	do {
 		ctrl_->integrateModels(s, gp_.modCount);
-		if ((res = solve.solve()) != value_true) { break; }
-		if (!ctrl_->commitModel(s))              { res = value_false; break; }
-	} 
-	act_ = 0;
+		up_ = act_ = 1; // activate enumerator and bounds
+		res = solve.solve();
+		up_ = act_ = 0; // de-activate enumerator and bounds
+		if      (res == value_true)  { term = !ctrl_->commitModel(s); }
+		else if (res == value_false) { term = !ctrl_->commitUnsat(s); solve.reset(term); gp_.reset(restart, gp_.type); }
+	} while (!term && res != value_free);
 	return res;
 }
 
@@ -756,13 +792,11 @@ void ParallelHandler::handleTerminateMessage() {
 // split-off new guiding path and add it to solve object
 void ParallelHandler::handleSplitMessage() {
 	assert(solver_ && "ParallelHandler::handleSplitMessage(): not attached!");
+	assert(solver_->splittable());
 	Solver& s = *solver_;
-	s.updateGuidingPath(gp_.path, gp_.pos, gp_.impl);
-	LitVec newGP(gp_.path);
-	s.pushRootLevel();
-	newGP.push_back(~s.decision(s.rootLevel()));
-	s.stats.addSplit();
-	ctrl_->pushWork(newGP);
+	SingleOwnerPtr<LitVec> newPath(new LitVec());
+	s.split(*newPath);
+	ctrl_->pushWork(newPath.release());
 }
 
 bool ParallelHandler::handleRestartMessage() {
@@ -794,8 +828,8 @@ bool ParallelHandler::propagateFixpoint(Solver& s, PostPropagator* ctx) {
 	// models/lemma exchange but only if path is setup.
 	// Skip updates if called from other post propagator so that we do not
 	// disturb any active propagation.
-	if (int up = (ctx == 0 && s.isTrue(s.sharedContext()->tagLiteral()) && up_ != 0)) {
-		up_ ^= s.strategy.upMode;
+	if (int up = (ctx == 0 && up_ != 0)) {
+		up_ ^= (uint32)s.updateMode();
 		up  += (act_ == 0 || (up_ && (s.stats.choices & 63) != 0));
 		if (s.stats.conflicts >= gp_.restart)  { ctrl_->requestRestart(); gp_.restart *= 2; }
 		for (uint32 cDL = s.decisionLevel();;) {
@@ -823,13 +857,13 @@ bool ParallelHandler::isModel(Solver& s) {
 		&& s.queueSize()   == 0;
 }
 bool ParallelHandler::integrate(Solver& s) {
-	uint32 rec = recEnd_ + s.sharedContext()->receive(s, received_ + recEnd_, RECEIVE_BUFFER_SIZE - recEnd_);
+	uint32 rec = recEnd_ + s.receive(received_ + recEnd_, RECEIVE_BUFFER_SIZE - recEnd_);
 	if (!rec) { return true; }
 	ClauseCreator::Result ret;
 	uint32 dl       = s.decisionLevel(), added = 0, i = 0;
 	uint32 intFlags = ctrl_->integrateFlags();
 	recEnd_         = 0;
-	if (s.strategy.updateLbd || params_->reduce.strategy.glue != 0) {
+	if (params_->reduce.strategy.glue != 0) {
 		intFlags |= ClauseCreator::clause_int_lbd;
 	}
 	do {
@@ -864,75 +898,77 @@ void ParallelHandler::add(ClauseHead* h) {
 	}
 }
 /////////////////////////////////////////////////////////////////////////////////////////
-// GlobalQueue
+// Distribution
 /////////////////////////////////////////////////////////////////////////////////////////
-GlobalQueue::GlobalQueue(const Policy& p, uint32 maxT, uint32 topo) : Distributor(p), queue_(0) {
-	assert(maxT <= ParallelSolveOptions::supportedSolvers());
-	queue_     = new Queue(maxT);
-	threadId_  = new ThreadInfo[maxT];
-	for (uint32 i = 0; i != maxT; ++i) {
-		threadId_[i].id       = queue_->addThread();
-		threadId_[i].peerMask = populatePeerMask(topo, maxT, i);
+uint64 ParallelSolveOptions::initPeerMask(uint32 id, Integration::Topology topo, uint32 maxT)  {
+	if (topo == Integration::topo_all) { return Distributor::initSet(maxT) ^ Distributor::mask(id); }
+	if (topo == Integration::topo_ring){
+		uint32 prev = id > 0 ? id - 1 : maxT - 1;
+		uint32 next = (id + 1) % maxT;
+		return Distributor::mask(prev) | Distributor::mask(next);
 	}
-}
-GlobalQueue::~GlobalQueue() {
-	release();
-}
-void GlobalQueue::release() {
-	if (queue_) {
-		for (uint32 i = 0; i != queue_->maxThreads(); ++i) {
-			Queue::ThreadId& id = getThreadId(i);
-			for (DistPair n; queue_->tryConsume(id, n); ) { 
-				if (n.sender != i) { n.lits->release(); }
-			}
-		}
-		delete queue_;
-		queue_ = 0;
-		delete [] threadId_;
-	}
-}
-uint64 GlobalQueue::populateFromCube(uint32 numThreads, uint32 myId, bool ext) const {
-	uint32 n = numThreads;
+	bool ext = topo == Integration::topo_cubex;
+	uint32 n = maxT;
 	uint32 k = 1;
 	for (uint32 i = n / 2; i > 0; i /= 2, k *= 2) { }
 	uint64 res = 0, x = 1;
 	for (uint32 m = 1; m <= k; m *= 2) {
-		uint32 i = m ^ myId;
+		uint32 i = m ^ id;
 		if      (i < n)         { res |= (x << i);     }
 		else if (ext && k != m) { res |= (x << (i^k)); }
 	}
 	if (ext) {
-		uint32 s = k ^ myId;
+		uint32 s = k ^ id;
 		for(uint32 m = 1; m < k && s >= n; m *= 2) {
 			uint32 i = m ^ s;
 			if (i < n) { res |= (x << i); }
 		}
 	}
-	assert( (res & (x<<myId)) == 0 );
+	assert( (res & (x<<id)) == 0 );
 	return res;
 }
-uint64 GlobalQueue::populatePeerMask(uint32 topo, uint32 maxT, uint32 id) const {
-	switch (topo) {
-		case ParallelSolveOptions::Integration::topo_ring: {
-			uint32 prev = id > 0 ? id - 1 : maxT - 1;
-			uint32 next = (id + 1) % maxT;
-			return Distributor::mask(prev) | Distributor::mask(next);
-		}
-		case ParallelSolveOptions::Integration::topo_cube:  return populateFromCube(maxT, id, false);
-		case ParallelSolveOptions::Integration::topo_cubex: return populateFromCube(maxT, id, true);
-		default:                                     return Distributor::initSet(maxT) ^ Distributor::mask(id);
+/////////////////////////////////////////////////////////////////////////////////////////
+// GlobalDistribution
+/////////////////////////////////////////////////////////////////////////////////////////
+GlobalDistribution::GlobalDistribution(const Policy& p, uint32 maxT, uint32 topo) : Distributor(p), queue_(0) {
+	typedef ParallelSolveOptions::Integration::Topology Topology;
+	assert(maxT <= ParallelSolveOptions::supportedSolvers());
+	Topology t = static_cast<Topology>(topo);
+	queue_     = new Queue(maxT);
+	threadId_  = (ThreadInfo*)alignedAlloc((maxT * sizeof(ThreadInfo)), 64);
+	for (uint32 i = 0; i != maxT; ++i) {
+		new (&threadId_[i]) ThreadInfo;
+		threadId_[i].id       = queue_->addThread();
+		threadId_[i].peerMask = ParallelSolveOptions::initPeerMask(i, t, maxT);
 	}
 }
-
-void GlobalQueue::publish(const Solver& s, SharedLiterals* n) {
-	assert(n->refCount() >= (queue_->maxThreads()-1));
-	queue_->publish(DistPair(s.id(), n), getThreadId(s.id()));
+GlobalDistribution::~GlobalDistribution() {
+	static_assert(sizeof(ThreadInfo) == 64, "Invalid size");
+	release();
 }
-uint32 GlobalQueue::receive(const Solver& in, SharedLiterals** out, uint32 maxn) {
+void GlobalDistribution::release() {
+	if (queue_) {
+		for (uint32 i = 0; i != queue_->maxThreads(); ++i) {
+			Queue::ThreadId& id = getThreadId(i);
+			for (ClausePair n; queue_->tryConsume(id, n); ) { 
+				if (n.sender != i) { n.lits->release(); }
+			}
+			threadId_[i].~ThreadInfo();
+		}
+		delete queue_;
+		queue_ = 0;
+		alignedFree(threadId_);
+	}
+}
+void GlobalDistribution::publish(const Solver& s, SharedLiterals* n) {
+	assert(n->refCount() >= (queue_->maxThreads()-1));
+	queue_->publish(ClausePair(s.id(), n), getThreadId(s.id()));
+}
+uint32 GlobalDistribution::receive(const Solver& in, SharedLiterals** out, uint32 maxn) {
 	uint32 r = 0;
 	Queue::ThreadId& id = getThreadId(in.id());
 	uint64 peers = getPeerMask(in.id());
-	for (DistPair n; r != maxn && queue_->tryConsume(id, n); ) {
+	for (ClausePair n; r != maxn && queue_->tryConsume(id, n); ) {
 		if (n.sender != in.id()) {
 			if (inSet(peers, n.sender))  { out[r++] = n.lits; }
 			else if (n.lits->size() == 1){ out[r++] = n.lits; }
@@ -941,6 +977,94 @@ uint32 GlobalQueue::receive(const Solver& in, SharedLiterals** out, uint32 maxn)
 	}
 	return r;
 }
+
+/////////////////////////////////////////////////////////////////////////////////////////
+// LocalDistribution
+/////////////////////////////////////////////////////////////////////////////////////////
+LocalDistribution::LocalDistribution(const Policy& p, uint32 maxT, uint32 topo) : Distributor(p), thread_(0), numThread_(0) {
+	typedef ParallelSolveOptions::Integration::Topology Topology;
+	assert(maxT <= ParallelSolveOptions::supportedSolvers());
+	Topology t = static_cast<Topology>(topo);
+	thread_    = new ThreadData*[numThread_ = maxT];
+	size_t sz  = ((sizeof(ThreadData) + 63) / 64) * 64;
+	for (uint32 i = 0; i != maxT; ++i) {
+		ThreadData* ti = new (alignedAlloc(sz, 64)) ThreadData;
+		ti->received.init(&ti->sentinal);
+		ti->peers = ParallelSolveOptions::initPeerMask(i, t, maxT);
+		ti->free  = 0;
+		thread_[i]= ti;
+	}
+}
+LocalDistribution::~LocalDistribution() {
+	while (numThread_) {
+		ThreadData* ti      = thread_[--numThread_];
+		thread_[numThread_] = 0;
+		for (QNode* n; (n = ti->received.pop()) != 0; ) {
+			static_cast<SharedLiterals*>(n->data)->release();
+		}
+		ti->~ThreadData();
+		alignedFree(ti);
+	}
+	for (MPSCPtrQueue::RawNode* n; (n = blocks_.tryPop()) != 0; ) {
+		alignedFree(n);
+	}
+	delete [] thread_;
+}
+
+void LocalDistribution::freeNode(uint32 tId, QNode* n) const {
+	if (n != &thread_[tId]->sentinal) {
+		n->next = thread_[tId]->free;
+		thread_[tId]->free = n;
+	}
+}
+
+LocalDistribution::QNode* LocalDistribution::allocNode(uint32 tId, SharedLiterals* clause) {
+	for (ThreadData* td = thread_[tId];;) {
+		if (QNode* n = td->free) {
+			td->free = static_cast<QNode*>(static_cast<MPSCPtrQueue::RawNode*>(n->next));
+			n->data  = clause;
+			return n;
+		}
+		// alloc a new block of node;
+		const uint32 nNodes = 128;
+		QNode* raw = (QNode*)alignedAlloc(sizeof(QNode) * nNodes, 64);
+		// add nodes [1, nNodes) to free list
+		for (uint32 i = 1; i != nNodes-1; ++i) {
+			raw[i].next = &raw[i+1];
+		}
+		raw[nNodes-1].next = 0;
+		td->free = &raw[1];
+		// use first node to link to block list
+		blocks_.push(raw);
+	}
+}
+
+void LocalDistribution::publish(const Solver& s, SharedLiterals* n) {
+	assert(n->refCount() >= (numThread_-1));
+	uint32 sender = s.id();
+	uint32 size   = n->size();
+	uint32 decRef = 0;
+	for (uint32 i = 0; i != numThread_; ++i) {
+		if (i == sender) { continue; }
+		if (size > 1 && !inSet(thread_[i]->peers, sender)) { ++decRef; }
+		else {
+			QNode* node = allocNode(sender, n);
+			thread_[i]->received.push(node);
+		}
+	}
+	if (decRef) { n->release(decRef); }
+}
+uint32 LocalDistribution::receive(const Solver& in, SharedLiterals** out, uint32 maxn) {
+	MPSCPtrQueue& q = thread_[in.id()]->received;
+	QNode*        n = 0;
+	uint32        r = 0;
+	while (r != maxn && (n = q.pop()) != 0) {
+		out[r++] = static_cast<SharedLiterals*>(n->data);
+		freeNode(in.id(), n);
+	}
+	return r;
+}
+
 } } // namespace Clasp::mt
 
 #endif
