@@ -104,8 +104,9 @@ bool SatBuilder::markAssigned() {
 void SatBuilder::prepareProblem(uint32 numVars, wsum_t cw, uint32 clauseHint) {
 	CLASP_ASSERT_CONTRACT_MSG(ctx(), "startProgram() not called!");
 	ctx()->resizeVars(numVars + 1);
-	ctx()->symbolTable().startInit();
-	ctx()->symbolTable().endInit(SymbolTable::map_direct, numVars+1);
+	ctx()->symbolTable().startInit(SymbolTable::map_direct);
+	ctx()->symbolTable().add(numVars+1);
+	ctx()->symbolTable().endInit();
 	ctx()->startAddConstraints(std::min(clauseHint, uint32(10000)));
 	varState_.resize(numVars + 1);
 	vars_       = ctx()->numVars();
@@ -114,16 +115,17 @@ void SatBuilder::prepareProblem(uint32 numVars, wsum_t cw, uint32 clauseHint) {
 }
 bool SatBuilder::addClause(LitVec& clause, wsum_t cw) {
 	if (!ctx()->ok() || satisfied(clause)) { return ctx()->ok(); }
-	CLASP_ASSERT_CONTRACT_MSG(cw >= 0 && cw <= std::numeric_limits<weight_t>::max(), "Clause weight out of bounds!");
-	if (cw == 0 && maxSat_) { cw = 1; }
-	if (cw == hardWeight_ || clause.empty()) {
+	CLASP_ASSERT_CONTRACT_MSG(cw >= 0 && (cw <= std::numeric_limits<weight_t>::max() || cw == hardWeight_), "Clause weight out of bounds!");
+	if (cw == 0 && maxSat_){ cw = 1; }
+	if (cw == hardWeight_) {
 		return ClauseCreator::create(*ctx()->master(), clause, Constraint_t::static_constraint).ok() && markAssigned();
 	}
 	else {
 		// Store weight, relaxtion var, and (optionally) clause
 		softClauses_.push_back(Literal::fromRep((uint32)cw));
-		if (clause.size() != 1) { softClauses_.push_back(posLit(++vars_)); softClauses_.insert(softClauses_.end(), clause.begin(), clause.end()); }
-		else                    { softClauses_.push_back(~clause.back());  }
+		if      (clause.size() > 1){ softClauses_.push_back(posLit(++vars_)); softClauses_.insert(softClauses_.end(), clause.begin(), clause.end()); }
+		else if (!clause.empty())  { softClauses_.push_back(~clause.back());  }
+		else                       { softClauses_.push_back(posLit(0)); }
 		softClauses_.back().watch(); // mark end of clause
 	}
 	return true;
@@ -191,8 +193,9 @@ void PBBuilder::prepareProblem(uint32 numVars, uint32 numProd, uint32 numSoft, u
 	nextVar_      = numVars;
 	maxVar_       = maxVar;
 	ctx()->resizeVars(maxVar + 1);
-	ctx()->symbolTable().startInit();
-	ctx()->symbolTable().endInit(SymbolTable::map_direct, numVars+1);
+	ctx()->symbolTable().startInit(SymbolTable::map_direct);
+	ctx()->symbolTable().add(numVars+1);
+	ctx()->symbolTable().endInit();
 	ctx()->startAddConstraints(numCons);
 }
 uint32 PBBuilder::getNextVar() {
