@@ -101,8 +101,8 @@ bool &ClingoLpOutput::disposeMinimize() {
 // {{{1 definition of ClingoControl
 
 #define LOG if (verbose_) std::cerr
-ClingoControl::ClingoControl(Gringo::GringoModule &module, bool clingoMode, Clasp::ClaspFacade *clasp, Clasp::Cli::ClaspCliConfig &claspConfig, PostGroundFunc pgf, PreSolveFunc psf)
-    : scripts(module)
+ClingoControl::ClingoControl(Gringo::Scripts &scripts, bool clingoMode, Clasp::ClaspFacade *clasp, Clasp::Cli::ClaspCliConfig &claspConfig, PostGroundFunc pgf, PreSolveFunc psf)
+    : scripts(scripts)
     , clasp(clasp)
     , claspConfig_(claspConfig)
     , pgf_(pgf)
@@ -137,6 +137,7 @@ void ClingoControl::parse(const StringSeq& files, const ClingoOptions& opts, Cla
         else          { lpOut.reset(new Output::PlainLparseOutputter(std::cout)); }
         out.reset(new Output::OutputBase(std::move(outPreds), *lpOut, opts.lparseDebug));
     }
+    out->keepFacts = opts.keepFacts;
     pb = gringo_make_unique<Input::NongroundProgramBuilder>(scripts, prg, *out, defs, opts.rewriteMinimize);
     parser = gringo_make_unique<Input::NonGroundParser>(*pb);
     for (auto &x : opts.defines) {
@@ -598,8 +599,8 @@ ClingoSolveFuture::~ClingoSolveFuture() { }
 
 // {{{1 definition of ClingoLib
 
-ClingoLib::ClingoLib(Gringo::GringoModule &module, int argc, char const **argv) 
-        : ClingoControl(module, true, &clasp_, claspConfig_, nullptr, nullptr) { 
+ClingoLib::ClingoLib(Gringo::Scripts &scripts, int argc, char const **argv) 
+        : ClingoControl(scripts, true, &clasp_, claspConfig_, nullptr, nullptr) { 
     using namespace ProgramOptions;
     OptionContext allOpts("<pyclingo>");
     initOptions(allOpts);
@@ -646,6 +647,7 @@ void ClingoLib::initOptions(ProgramOptions::OptionContext& root) {
          "      [no-]variable-unbounded:    $x > 10.\n"
          "      [no-]global-variable:       :- #count { X } = 1, X = 1.\n")
         ("rewrite-minimize"         , flag(grOpts_.rewriteMinimize = false), "Rewrite minimize constraints into rules")
+        ("keep-facts"               , flag(grOpts_.keepFacts = false), "Do not remove facts from normal rules")
         ;
     root.add(gringo);
     claspConfig_.addOptions(root);
@@ -683,14 +685,17 @@ ClingoLib::~ClingoLib() {
     clasp_.shutdown();
 }
 
-// }}}1
-// {{{ definition of DefaultGringoModule
+// {{{1 definition of DefaultGringoModule
 
+DefaultGringoModule::DefaultGringoModule()
+: scripts(*this) { }
 Gringo::Control *DefaultGringoModule::newControl(int argc, char const **argv) {
-    return new ClingoLib(*this, argc, argv);
+    return new ClingoLib(scripts, argc, argv);
 }
 
 void DefaultGringoModule::freeControl(Gringo::Control *ctl) {
     if (ctl) { delete ctl; }
 }
 Gringo::Value DefaultGringoModule::parseValue(std::string const &str) { return parser.parse(str); }
+
+// }}}1
