@@ -23,6 +23,7 @@
 #include "gringo/logger.hh"
 #include "gringo/scripts.hh"
 #include <cmath>
+#include <cstring>
 
 namespace Gringo { namespace Ground {
 
@@ -433,25 +434,30 @@ Literal::Score CSPLiteral::score(Term::VarSet const &) {
 // }}}
 // {{{ definition of *Literal::toOutput
 
-Output::Literal *RangeLiteral::toOutput()     { return nullptr; }
-Output::Literal *ScriptLiteral::toOutput()    { return nullptr; }
-Output::Literal *RelationLiteral::toOutput()  { return nullptr; }
-Output::Literal *PredicateLiteral::toOutput() { 
+std::pair<Output::Literal*,bool> RangeLiteral::toOutput()     { return {nullptr, true}; }
+std::pair<Output::Literal*,bool> ScriptLiteral::toOutput()    { return {nullptr, true}; }
+std::pair<Output::Literal*,bool> RelationLiteral::toOutput()  { return {nullptr, true}; }
+std::pair<Output::Literal*,bool> PredicateLiteral::toOutput() { 
+    // TODO: the name isFalse is really bad...
+    // TODO: make the inc_ literals a decicated class
+    if (gLit.repr->second.isFalse() || std::strncmp("#inc_", gLit.repr->first.name()->c_str(), 5) == 0) {
+        return {nullptr,true};
+    }
     switch (gLit.naf) {
         case NAF::POS:
-        case NAF::NOTNOT: { return !gLit.repr->second.fact(isRecursive()) ? &gLit : nullptr; }
-        case NAF::NOT:    { return !gLit.repr->second.isFalse() ? &gLit : nullptr; }
+        case NAF::NOTNOT: { return {&gLit, gLit.repr->second.fact(isRecursive())}; }
+        case NAF::NOT:    { return {&gLit, gLit.repr->second.isFalse()}; }
     }
     assert(false);
-    return nullptr;
+    return {nullptr,true};
 }
-Output::Literal *CSPLiteral::toOutput() {
+std::pair<Output::Literal*,bool> CSPLiteral::toOutput() {
     CSPGroundLit add;
     std::get<0>(add) = std::get<0>(terms);
     std::get<1>(terms).toGround(add, false);
     std::get<2>(terms).toGround(add, true);
     repr.reset(std::move(add));
-    return &repr;
+    return {&repr, false};
 }
 
 // }}}
