@@ -1,4 +1,4 @@
-// {{{ GPL License 
+// {{{ GPL License
 
 // This file is part of gringo - a grounder for logic programs.
 // Copyright (C) 2013  Roland Kaminski
@@ -65,7 +65,7 @@ void _analyze(Statement::Dep::Node &node, Statement::Dep &dep, ULitVec const &li
         if (occ) { dep.depends(node, *occ, forceNegative); }
     }
 }
-    
+
 // {{{2 definition of _linearize (Note: could be moved into AbstractStatement)
 
 struct Ent {
@@ -87,7 +87,7 @@ InstVec _linearize(Scripts &scripts, bool positive, SolutionCallback &cb, Term::
         if (x->isRecursive() && x->occurrence() && !x->occurrence()->isNegative()) { rec.emplace_back(i); }
         ++i;
     }
-    for (auto &x : aux) { 
+    for (auto &x : aux) {
         todo.back().emplace_back(BinderType::ALL, x.get());
         if (x->isRecursive() && x->occurrence() && !x->occurrence()->isNegative()) { rec.emplace_back(i); }
         ++i;
@@ -96,7 +96,7 @@ InstVec _linearize(Scripts &scripts, bool positive, SolutionCallback &cb, Term::
     insts.reserve(todo.capacity()); // Note: preserve references
     for (auto i : rec) {
         todo.back()[i].first = BinderType::NEW;
-        if (i != rec.back()) { 
+        if (i != rec.back()) {
             todo.emplace_back(todo.back());
             todo.back()[i].first = BinderType::OLD;
         }
@@ -115,7 +115,7 @@ InstVec _linearize(Scripts &scripts, bool positive, SolutionCallback &cb, Term::
             lit.second->collect(vars);
             for (auto &occ : vars) {
                 auto &varNode(varMap[occ.first->name]);
-                if (!varNode)   { 
+                if (!varNode)   {
                     varNode = &s.insertVar(boundBy.size());
                     boundBy.emplace_back(occ.first->name, std::vector<unsigned>{});
                 }
@@ -183,7 +183,7 @@ struct BindOnce : Binder, IndexUpdater {
     IndexUpdater *getUpdater() { return this; }
     bool update() { return true; }
     void match() { once = true; }
-    bool next() { 
+    bool next() {
         bool ret = once;
         once = false;
         return ret;
@@ -268,15 +268,15 @@ void AbstractStatement::analyze(Dep::Node &node, Dep &dep) {
   _analyze(node, dep, auxLits);
 }
 
-void AbstractStatement::startLinearize(bool active) { 
+void AbstractStatement::startLinearize(bool active) {
     if (def.repr) { def.active = active; }
     if (active) { insts.clear(); }
 }
 
-void AbstractStatement::collectImportant(Term::VarSet &vars) { 
+void AbstractStatement::collectImportant(Term::VarSet &vars) {
     if (def.repr) { def.collectImportant(vars); }
 }
-void AbstractStatement::linearize(Scripts &scripts, bool positive) { 
+void AbstractStatement::linearize(Scripts &scripts, bool positive) {
     Term::VarSet important;
     collectImportant(important);
     insts = _linearize(scripts, positive, *this, std::move(important), lits, auxLits);
@@ -335,14 +335,19 @@ void Rule::report(Output::OutputBase &out) {
         case RuleType::NORMAL: {
             Output::RuleRef &rule(out.tempRule);
             rule.body.clear();
+            bool fact = true;
             for (auto &x : lits) {
-                if (auto lit = x->toOutput()) { rule.body.emplace_back(*lit); }
+                auto ret = x->toOutput();
+                if (ret.first && (out.keepFacts || !ret.second)) {
+                    rule.body.emplace_back(*ret.first);
+                }
+                if (!ret.second) { fact = false; }
             }
             if (def.repr) {
                 bool undefined = false;
                 Value val = def.repr->eval(undefined);
                 if (!undefined) {
-                    auto ret(static_cast<PredicateDomain*>(def.domain)->insert(val, rule.body.empty()));
+                    auto ret(static_cast<PredicateDomain*>(def.domain)->insert(val, fact));
                     if (!std::get<2>(ret)) {
                         rule.head = std::get<0>(ret);
                         out.output(rule);
@@ -358,7 +363,7 @@ void Rule::report(Output::OutputBase &out) {
     }
 }
 
-bool Rule::isNormal() const { return def.repr && type != RuleType::EXTERNAL; } 
+bool Rule::isNormal() const { return def.repr && type != RuleType::EXTERNAL; }
 
 void Rule::printHead(std::ostream &out) const {
     if (type == RuleType::EXTERNAL) { out << "#external "; }
@@ -368,7 +373,7 @@ void Rule::printHead(std::ostream &out) const {
 void Rule::print(std::ostream &out) const {
     assert(auxLits.empty());
     printHead(out);
-    if (!lits.empty()) { 
+    if (!lits.empty()) {
         out << (type == RuleType::EXTERNAL ? ":" : ":-");
         printBody(out);
     }
@@ -388,17 +393,18 @@ void WeakConstraint::report(Output::OutputBase &out) {
     if (!undefined && out.tempVals.front().type() == Value::NUM) {
         Output::ULitVec cond;
         for (auto &x : lits) {
-            if (auto lit = x->toOutput()) { cond.emplace_back(lit->clone()); }
+            auto lit = x->toOutput();
+            if (!lit.second) { cond.emplace_back(lit.first->clone()); }
         }
         Output::Minimize min;
         min.elems.emplace_back(
-                std::piecewise_construct, 
-                std::forward_as_tuple(out.tempVals), 
+                std::piecewise_construct,
+                std::forward_as_tuple(out.tempVals),
                 std::forward_as_tuple(std::move(cond)));
         out.output(min);
     }
     else if (!undefined) {
-        GRINGO_REPORT(W_OPERATION_UNDEFINED) 
+        GRINGO_REPORT(W_OPERATION_UNDEFINED)
             << tuple.front()->loc() << ": info: tuple ignored:\n"
             << "  " << out.tempVals.front() << "\n";
     }
@@ -436,7 +442,7 @@ void ExternalRule::linearize(Scripts &, bool) { }
 void ExternalRule::enqueue(Queue &) {
 }
 void ExternalRule::print(std::ostream &out) const {
-    out << "#external."; 
+    out << "#external.";
 }
 ExternalRule::~ExternalRule()                                   { }
 
@@ -570,10 +576,11 @@ void BodyAggregateAccumulate::report(Output::OutputBase &out) {
     if (!undefined) {
         out.tempLits.clear();
         for (auto &x : lits) {
-            if (auto lit = x->toOutput()) { out.tempLits.emplace_back(*lit); }
+            auto lit = x->toOutput();
+            if (!lit.second) { out.tempLits.emplace_back(*lit.first); }
         }
         auto state = complete.domain.domain.find(repr);
-        if (state == complete.domain.domain.end()) { 
+        if (state == complete.domain.domain.end()) {
             state = complete.domain.domain.emplace(std::piecewise_construct, std::forward_as_tuple(repr), std::forward_as_tuple()).first;
         }
         if (!state->second.initialized) {
@@ -612,8 +619,8 @@ void BodyAggregateAccumulate::printHead(std::ostream &out) const {
 
 // {{{3 definition of BodyAggregateLiteral
 
-BodyAggregateLiteral::BodyAggregateLiteral(BodyAggregateComplete &complete, NAF naf) 
-: complete(complete) { 
+BodyAggregateLiteral::BodyAggregateLiteral(BodyAggregateComplete &complete, NAF naf)
+: complete(complete) {
     gLit.naf = naf;
 }
 BodyAggregateLiteral::~BodyAggregateLiteral() { }
@@ -627,13 +634,13 @@ void BodyAggregateLiteral::checkDefined(LocSet &, SigSet const &, UndefVec &) co
 void BodyAggregateLiteral::print(std::ostream &out) const {
     out << gLit.naf;
     auto it = std::begin(complete.bounds), ie = std::end(complete.bounds);
-    if (it != ie) { 
+    if (it != ie) {
         it->bound->print(out);
         out << inv(it->rel);
         ++it;
     }
     out << complete.fun << "{" << *complete.def.repr << type << "}";
-    if (it != ie) { 
+    if (it != ie) {
         out << it->rel;
         it->bound->print(out);
     }
@@ -645,7 +652,7 @@ UIdx BodyAggregateLiteral::index(Scripts &, BinderType type, Term::VarSet &bound
     return make_binder(complete.domain, gLit.naf, *complete.def.repr, gLit.repr, type, isRecursive(), bound, 0);
 }
 Literal::Score BodyAggregateLiteral::score(Term::VarSet const &bound) { return gLit.naf == NAF::POS ? estimate(complete.domain.exports.size(), *complete.def.repr, bound) : 0; }
-Output::Literal *BodyAggregateLiteral::toOutput() { 
+std::pair<Output::Literal*,bool> BodyAggregateLiteral::toOutput() {
     gLit.incomplete = isRecursive();
     gLit.fun        = complete.fun;
     gLit.bounds.clear();
@@ -654,11 +661,11 @@ Output::Literal *BodyAggregateLiteral::toOutput() {
     assert(!undefined);
     switch (gLit.naf) {
         case NAF::POS:
-        case NAF::NOTNOT: { return !gLit.repr->second.fact(gLit.incomplete) ? &gLit : nullptr; }
-        case NAF::NOT:    { return gLit.incomplete || !gLit.repr->second.isFalse() ? &gLit : nullptr; }
+        case NAF::NOTNOT: { return {&gLit, gLit.repr->second.fact(gLit.incomplete)}; }
+        case NAF::NOT:    { return {&gLit, !gLit.incomplete && gLit.repr->second.isFalse()}; }
     }
     assert(false);
-    return nullptr;
+    return {nullptr, true};
 }
 
 // {{{2 AssignmentAggregate
@@ -849,7 +856,8 @@ void AssignmentAggregateAccumulate::report(Output::OutputBase &out) {
     if (!undefined) {
         out.tempLits.clear();
         for (auto &x : lits) {
-            if (auto lit = x->toOutput()) { out.tempLits.emplace_back(*lit); }
+            auto lit = x->toOutput();
+            if (!lit.second) { out.tempLits.emplace_back(*lit.first); }
         }
         auto &data = *complete.data.emplace(std::piecewise_construct, std::forward_as_tuple(dataRepr), std::forward_as_tuple()).first;
         if (!Output::neutral(out.tempVals, complete.fun, tuple.empty() ? def.repr->loc() : tuple.front()->loc())) {
@@ -874,7 +882,7 @@ void AssignmentAggregateAccumulate::printHead(std::ostream &out) const {
 
 // {{{3 Definition of AssignmentAggregateLiteral
 
-AssignmentAggregateLiteral::AssignmentAggregateLiteral(AssignmentAggregateComplete &complete) 
+AssignmentAggregateLiteral::AssignmentAggregateLiteral(AssignmentAggregateComplete &complete)
 : complete(complete) { }
 AssignmentAggregateLiteral::~AssignmentAggregateLiteral() { }
 UGTerm AssignmentAggregateLiteral::getRepr() const { return complete.def.repr->gterm(); }
@@ -892,10 +900,10 @@ UIdx AssignmentAggregateLiteral::index(Scripts &, BinderType type, Term::VarSet
     return make_binder(complete.domain, NAF::POS, *complete.def.repr, gLit.repr, type, isRecursive(), bound, 0);
 }
 Literal::Score AssignmentAggregateLiteral::score(Term::VarSet const &bound) { return estimate(complete.domain.exports.size(), *complete.def.repr, bound); }
-Output::Literal *AssignmentAggregateLiteral::toOutput() { 
+std::pair<Output::Literal*,bool> AssignmentAggregateLiteral::toOutput() {
     gLit.incomplete = isRecursive();
     gLit.fun        = complete.fun;
-    return !gLit.repr->second.fact(gLit.incomplete) ? &gLit : nullptr;
+    return {&gLit, gLit.repr->second.fact(gLit.incomplete)};
 }
 ///}}}3
 
@@ -903,7 +911,7 @@ Output::Literal *AssignmentAggregateLiteral::toOutput() {
 
 // {{{3 definition of ConjunctionLiteral
 
-ConjunctionLiteral::ConjunctionLiteral(ConjunctionComplete &complete) 
+ConjunctionLiteral::ConjunctionLiteral(ConjunctionComplete &complete)
     : complete(complete) { }
 
 ConjunctionLiteral::~ConjunctionLiteral() = default;
@@ -932,10 +940,10 @@ BodyOcc::DefinedBy &ConjunctionLiteral::definedBy() {
     return defs;
 }
 
-void ConjunctionLiteral::checkDefined(LocSet &, SigSet const &, UndefVec &) const { 
+void ConjunctionLiteral::checkDefined(LocSet &, SigSet const &, UndefVec &) const {
 }
 
-void ConjunctionLiteral::print(std::ostream &out) const { 
+void ConjunctionLiteral::print(std::ostream &out) const {
     out << *complete.def.repr << type;
 }
 
@@ -959,15 +967,15 @@ Literal::Score ConjunctionLiteral::score(Term::VarSet const &bound) {
     return estimate(complete.dom.exports.size(), *complete.def.repr, bound);
 }
 
-Output::Literal *ConjunctionLiteral::toOutput() { 
-    return !gLit.repr->second.fact(gLit.repr->second.incomplete) ? &gLit : nullptr;
+std::pair<Output::Literal*,bool> ConjunctionLiteral::toOutput() {
+    return {&gLit, gLit.repr->second.fact(gLit.repr->second.incomplete)};
 }
 
 // {{{3 definition of ConjunctionAccumulateEmpty
 
 ConjunctionAccumulateEmpty::ConjunctionAccumulateEmpty(ConjunctionComplete &complete, ULitVec &&auxLits)
 : AbstractStatement(complete.emptyRepr(), &complete.domEmpty, {}, std::move(auxLits))
-, complete(complete) { 
+, complete(complete) {
 }
 
 ConjunctionAccumulateEmpty::~ConjunctionAccumulateEmpty() = default;
@@ -990,7 +998,7 @@ void ConjunctionAccumulateEmpty::report(Output::OutputBase &) {
 
 // {{{3 definition of ConjunctionAccumulateCond
 
-ConjunctionAccumulateCond::ConjunctionAccumulateCond(ConjunctionComplete &complete, ULitVec &&lits) 
+ConjunctionAccumulateCond::ConjunctionAccumulateCond(ConjunctionComplete &complete, ULitVec &&lits)
 : AbstractStatement(complete.condRepr(), &complete.domCond, std::move(lits), {})
 , complete(complete) {
     auxLits.emplace_back(gringo_make_unique<PredicateLiteral>(complete.domEmpty, NAF::POS, complete.emptyRepr()));
@@ -1015,9 +1023,10 @@ void ConjunctionAccumulateCond::report(Output::OutputBase &) {
 
     auto &state = *complete.dom.domain.emplace(std::piecewise_construct, std::forward_as_tuple(litRepr), std::forward_as_tuple()).first;
     Output::ULitVec cond;
-    for (auto &x : lits) { 
-        if(auto y = x->toOutput()) { 
-            cond.emplace_back(y->clone());
+    for (auto &x : lits) {
+        auto y = x->toOutput();
+        if (!y.second) {
+            cond.emplace_back(y.first->clone());
         }
     }
     complete.domCond.insert(condRepr, cond.empty());
@@ -1040,7 +1049,7 @@ void ConjunctionAccumulateCond::report(Output::OutputBase &) {
 
 // {{{3 definition of ConjunctionAccumulateHead
 
-ConjunctionAccumulateHead::ConjunctionAccumulateHead(ConjunctionComplete &complete, ULitVec &&lits) 
+ConjunctionAccumulateHead::ConjunctionAccumulateHead(ConjunctionComplete &complete, ULitVec &&lits)
 : AbstractStatement(complete.headRepr(), nullptr, std::move(lits), {})
 , complete(complete) {
     auxLits.emplace_back(gringo_make_unique<PredicateLiteral>(complete.domCond, NAF::POS, complete.condRepr()));
@@ -1060,12 +1069,13 @@ void ConjunctionAccumulateHead::report(Output::OutputBase &) {
 
     auto &state = *complete.dom.domain.find(litRepr);
     Output::ULitVec head;
-    for (auto &x : lits) { 
-        if(auto y = x->toOutput()) {
-            head.emplace_back(y->clone());
+    for (auto &x : lits) {
+        auto y = x->toOutput();
+        if (!y.second) {
+            head.emplace_back(y.first->clone());
         }
     }
-    
+
     auto &elem = *state.second.elems.find(condRepr.args()[2]);
     if (elem.heads.empty() && elem.bodies.size() == 1 && elem.bodies.back().empty()) { --state.second.numBlocked; }
     if (head.empty()) { elem.heads.clear(); }
@@ -1134,7 +1144,7 @@ void ConjunctionComplete::linearize(Scripts &, bool){
     for (HeadOccurrence &x : defBy) { x.defines(*binder->getUpdater(), &inst); }
     inst.add(std::move(binder), Instantiator::DependVec{});
     inst.finalize(Instantiator::DependVec{});
-} 
+}
 
 void ConjunctionComplete::enqueue(Queue &q){
     dom.init();
@@ -1143,7 +1153,7 @@ void ConjunctionComplete::enqueue(Queue &q){
 
 void ConjunctionComplete::printHead(std::ostream &out) const{
     out << *def.repr;
-} 
+}
 
 void ConjunctionComplete::propagate(Queue &queue) {
     def.enqueue(queue);
@@ -1152,16 +1162,16 @@ void ConjunctionComplete::propagate(Queue &queue) {
 void ConjunctionComplete::report(Output::OutputBase &) {
     // NOTE: this code relies on priority set to 2!
     // NOTE about handling incompleteness:
-    //   - if both the condition and the head are stratified 
+    //   - if both the condition and the head are stratified
     //     then the conjunction is complete
-    //   - if the condition is stratified but the head is recursive, 
+    //   - if the condition is stratified but the head is recursive,
     //     then the conjunction is complete if all heads are fact
     for (DomainElement &x : todo) {
         if (x.second.numBlocked == 0) {
             if (!condRecursive) {
                 x.second._fact = true;
                 for (auto &y : x.second.elems) {
-                    if (y.bodies.size() != 1 || !y.bodies.front().empty() || 
+                    if (y.bodies.size() != 1 || !y.bodies.front().empty() ||
                         y.heads.size()  != 1 || !y.heads.front().empty()
                     ) {
                         x.second._fact = false;
@@ -1178,9 +1188,9 @@ void ConjunctionComplete::report(Output::OutputBase &) {
         else { x.second.dequeue(); }
     }
     todo.clear();
-} 
+}
 
-void ConjunctionComplete::print(std::ostream &out) const{ 
+void ConjunctionComplete::print(std::ostream &out) const{
     printHead(out);
     out << ":-" << *accuRepr();
 }
@@ -1224,7 +1234,7 @@ DisjointComplete::DisjointComplete(UTerm &&repr)
 , inst(*this) { }
 
 bool DisjointComplete::isNormal() const {
-    return true;
+    return false;
 }
 void DisjointComplete::analyze(Dep::Node &node, Dep &dep) {
     dep.depends(node, *this);
@@ -1334,14 +1344,15 @@ void DisjointAccumulate::report(Output::OutputBase &out) {
             auto &elem = *state.second.elems.emplace_back(std::piecewise_construct, std::forward_as_tuple(out.tempVals), std::forward_as_tuple()).first;
             Output::ULitVec outLits;
             for (auto &x : lits) {
-                if (auto lit = x->toOutput()) { 
-                    outLits.emplace_back(lit->clone());
+                auto lit = x->toOutput();
+                if (!lit.second) {
+                    outLits.emplace_back(lit.first->clone());
                 }
             }
             elem.second.emplace_back(std::move(std::get<1>(gVal)), -std::get<2>(gVal), std::move(outLits));
         }
     }
-    if (!state.second.enqueued) {
+    if (!state.second.enqueued && !state.second.defined()) {
         complete.todo.emplace_back(state);
         state.second.enqueued = true;
     }
@@ -1362,7 +1373,7 @@ void DisjointAccumulate::printHead(std::ostream &out) const {
 
 // {{{3 definition of DisjointLiteral
 
-DisjointLiteral::DisjointLiteral(DisjointComplete &complete, NAF naf) 
+DisjointLiteral::DisjointLiteral(DisjointComplete &complete, NAF naf)
 : complete(complete)
 , gLit(naf) { }
 DisjointLiteral::~DisjointLiteral() { }
@@ -1381,9 +1392,9 @@ UIdx DisjointLiteral::index(Scripts &, BinderType type, Term::VarSet &bound) {
     return make_binder(complete.domain, gLit.naf, *complete.def.repr, gLit.repr, type, isRecursive(), bound, 0);
 }
 Literal::Score DisjointLiteral::score(Term::VarSet const &bound) { return gLit.naf == NAF::POS ? estimate(complete.domain.exports.size(), *complete.def.repr, bound) : 0; }
-Output::Literal *DisjointLiteral::toOutput() { 
+std::pair<Output::Literal*,bool> DisjointLiteral::toOutput() {
     gLit.incomplete = isRecursive();
-    return &gLit;
+    return {&gLit, false};
 }
 
 // {{{1 Head Aggregates
@@ -1401,7 +1412,10 @@ void HeadAggregateRule::report(Output::OutputBase &out) {
     std::unique_ptr<Output::HeadAggregateRule> rule(gringo_make_unique<Output::HeadAggregateRule>());
     rule->fun = fun;
     for (auto &x : lits) {
-        if (auto lit = x->toOutput()) { rule->body.emplace_back(Output::ULit(lit->clone())); }
+        auto ret = x->toOutput();
+        if (ret.first && (out.keepFacts || !ret.second)) {
+            rule->body.emplace_back(Output::ULit(ret.first->clone()));
+        }
     }
     bool undefined = false;
     for (auto &x : bounds) { rule->bounds.emplace_back(x.rel, x.bound->eval(undefined)); }
@@ -1412,7 +1426,7 @@ void HeadAggregateRule::report(Output::OutputBase &out) {
         domain.exports.append(*ret.first);
     }
     rule->repr = &ret.first->second;
-    // NOTE: the head here might be false 
+    // NOTE: the head here might be false
     //       it does not make much sense to output the unsatisfiable aggregate head in this case
     //       the delayed output should take this into consideration...
     out.output(std::move(rule));
@@ -1463,7 +1477,8 @@ void HeadAggregateAccumulate::report(Output::OutputBase &out) {
     for (auto &x : tuple) { out.tempVals.emplace_back(x->eval(undefined)); }
     out.tempLits.clear();
     for (auto &x : lits) {
-        if (auto lit = x->toOutput()) { out.tempLits.emplace_back(*lit); }
+        auto lit = x->toOutput();
+        if (!lit.second) { out.tempLits.emplace_back(*lit.first); }
     }
     if (!undefined) {
         Value predVal(predDef ? predDef->repr->eval(undefined) : Value());
@@ -1483,7 +1498,7 @@ void HeadAggregateAccumulate::report(Output::OutputBase &out) {
 
 void HeadAggregateAccumulate::printHead(std::ostream &out) const {
     out << "#accu(" << *headRule.def.repr << ",";
-    if (predDef) { 
+    if (predDef) {
         out << *predDef->repr << ",tuple(" << tuple << ")";
     }
     else { out << "#true"; }
@@ -1492,7 +1507,7 @@ void HeadAggregateAccumulate::printHead(std::ostream &out) const {
 
 // {{{3 definition of HeadAggregateComplete
 
-HeadAggregateComplete::HeadAggregateComplete(HeadAggregateRule &headRule) 
+HeadAggregateComplete::HeadAggregateComplete(HeadAggregateRule &headRule)
     : headRule(headRule)
     , completeRepr(completeRepr_(headRule.def.repr))
     , inst(*this) { }
@@ -1503,13 +1518,13 @@ void HeadAggregateComplete::analyze(Dep::Node &node, Dep &dep) {
     }
     dep.depends(node, *this);
 }
-void HeadAggregateComplete::startLinearize(bool active) { 
+void HeadAggregateComplete::startLinearize(bool active) {
     for (HeadAggregateAccumulate &x : accuDoms) {
         if (x.predDef) { x.predDef->active = active; }
     }
     if (active) { inst = Instantiator(*this); }
 }
-void HeadAggregateComplete::linearize(Scripts &, bool) { 
+void HeadAggregateComplete::linearize(Scripts &, bool) {
     auto binder  = gringo_make_unique<BindOnce>();
     for (HeadOccurrence &x : defBy) { x.defines(*binder->getUpdater(), &inst); }
     inst.add(std::move(binder), Instantiator::DependVec{});
@@ -1532,8 +1547,8 @@ void HeadAggregateComplete::printHead(std::ostream &out) const {
     print_comma(out, accuDoms, ";", [](std::ostream &out, HeadAggregateAccumulate const &x) -> void {
         out << x.tuple;
         out << ":";
-        if (x.predDef) { 
-            out << x.predDef->repr; 
+        if (x.predDef) {
+            out << x.predDef->repr;
         } else {
             out << "#true";
         }
@@ -1546,7 +1561,7 @@ void HeadAggregateComplete::printHead(std::ostream &out) const {
         out << it->rel;
         out << *it->bound;
     }
-}         
+}
 void HeadAggregateComplete::propagate(Queue &queue) {
     for (HeadAggregateAccumulate &x : accuDoms) {
         if (x.predDef) { x.predDef->enqueue(queue); }
@@ -1563,7 +1578,7 @@ void HeadAggregateComplete::report(Output::OutputBase &) {
         if (x.bounds.intersects(x.range(headRule.fun))) {
             for (auto &y : x.elems) {
                 for (auto it(y.second.conds.begin() + y.second.imported), ie(y.second.conds.end()); it != ie; ++it) {
-                    if (it->head) { 
+                    if (it->head) {
                         static_cast<PredicateDomain*>(accuDoms[it->headNum].get().predDef->domain)->insert(*it->head);
                     }
                 }
@@ -1602,7 +1617,9 @@ UIdx HeadAggregateLiteral::index(Scripts &, BinderType type, Term::VarSet &bound
     return make_binder(headRule.domain, NAF::POS, *headRule.def.repr, gResult, type, isRecursive(), bound, 0);
 }
 Literal::Score HeadAggregateLiteral::score(Term::VarSet const &bound) { return estimate(headRule.domain.exports.size(), *headRule.def.repr, bound); }
-Output::Literal *HeadAggregateLiteral::toOutput() { return nullptr; }
+std::pair<Output::Literal*,bool> HeadAggregateLiteral::toOutput() {
+    return {nullptr, true};
+}
 
 // {{{2 Disjunction
 
@@ -1610,7 +1627,7 @@ Output::Literal *HeadAggregateLiteral::toOutput() { return nullptr; }
 
 DisjunctionRule::DisjunctionRule(DisjunctionComplete &complete, ULitVec &&lits)
 : AbstractStatement(complete.emptyRepr(), &complete.domEmpty, std::move(lits), {})
-, complete(complete) { 
+, complete(complete) {
 }
 
 bool DisjunctionRule::isNormal() const {
@@ -1632,7 +1649,10 @@ void DisjunctionRule::report(Output::OutputBase &out) {
 
     std::unique_ptr<Output::DisjunctionRule> rule(gringo_make_unique<Output::DisjunctionRule>());
     for (auto &x : lits) {
-        if (auto lit = x->toOutput()) { rule->body.emplace_back(Output::ULit(lit->clone())); }
+        auto ret = x->toOutput();
+        if (ret.first && (out.keepFacts || !ret.second)) {
+            rule->body.emplace_back(Output::ULit(ret.first->clone()));
+        }
     }
     rule->repr = &state.second;
     out.output(std::move(rule));
@@ -1656,9 +1676,10 @@ void DisjunctionAccumulateCond::report(Output::OutputBase &) {
 
     auto &state = *complete.dom.emplace(std::piecewise_construct, std::forward_as_tuple(litRepr), std::forward_as_tuple()).first;
     Output::ULitVec cond;
-    for (auto &x : lits) { 
-        if(auto y = x->toOutput()) { 
-            cond.emplace_back(y->clone());
+    for (auto &x : lits) {
+        auto y = x->toOutput();
+        if (!y.second) {
+            cond.emplace_back(y.first->clone());
         }
     }
     complete.domCond.insert(condRepr, cond.empty());
@@ -1695,9 +1716,10 @@ void DisjunctionAccumulateHead::report(Output::OutputBase &) {
         auto &cond = *state.second.elems.find(headRepr.args()[2]);
         if (cond.heads.size() != 1 || !cond.heads.front().empty()) {
             Output::ULitVec head;
-            for (auto &x : lits) { 
-                if (auto y = x->toOutput()) {
-                    auto z = get_clone(y);
+            for (auto &x : lits) {
+                auto y = x->toOutput();
+                if (!y.second) {
+                    auto z = get_clone(y.first);
                     assert(z->invertible());
                     z->invert();
                     head.emplace_back(std::move(z));
@@ -1790,7 +1812,7 @@ void DisjunctionComplete::linearize(Scripts &, bool) {
     for (HeadOccurrence &x : defBy) { x.defines(*binder->getUpdater(), &inst); }
     inst.add(std::move(binder), Instantiator::DependVec{});
     inst.finalize(Instantiator::DependVec{});
-} 
+}
 
 void DisjunctionComplete::enqueue(Queue &q) {
     for (HeadDefinition &x : heads) {
@@ -1827,9 +1849,9 @@ void DisjunctionComplete::report(Output::OutputBase &) {
                 auto h = static_cast<PredicateDomain*>(heads[headIndex].domain)->insert(std::get<2>(todo), false);
                 head.emplace_back(gringo_make_unique<Output::PredicateLiteral>(NAF::POS, *std::get<0>(h)));
             }
-            if (head.empty()) { 
+            if (head.empty()) {
                 // NOTE: the head is a conjunction of disjunctions
-                cond.heads.clear(); 
+                cond.heads.clear();
             }
             cond.heads.emplace_back(std::move(head));
         }
@@ -1840,9 +1862,9 @@ void DisjunctionComplete::report(Output::OutputBase &) {
         x.generation(1);
     }
     todo.clear();
-} 
+}
 
-void DisjunctionComplete::print(std::ostream &out) const{ 
+void DisjunctionComplete::print(std::ostream &out) const{
     printHead(out);
     out << ":-" << *accuRepr() << occType;
 }
