@@ -70,7 +70,7 @@ bool CheckLevel::check() {
         std::ostringstream msg;
         msg << loc << ": error: unsafe variables in:\n  " << p << "\n";
         for (auto &x : vars) { msg << x->data->loc() << ": note: '" << x->data->name << "' is unsafe\n"; }
-        GRINGO_REPORT(ERROR) << msg.str();
+        GRINGO_REPORT(E_ERROR) << msg.str();
     }
     return vars.empty();
 }
