@@ -571,22 +571,6 @@ UHeadAggr SimpleHeadLiteral::rewriteAggregates(UBodyAggrVec &aggr) {
     return nullptr;
 }
 
-// }}}
-// {{{ definition of Aggregate::shiftHead
-
-UHeadAggr TupleHeadAggregate::shiftHead(UBodyAggrVec &) { 
-    return nullptr;
-}
-UHeadAggr LitHeadAggregate::shiftHead(UBodyAggrVec &) {
-    return nullptr;
-}
-UHeadAggr Disjunction::shiftHead(UBodyAggrVec &) { 
-    return nullptr;
-}
-UHeadAggr SimpleHeadLiteral::shiftHead(UBodyAggrVec &) { 
-    return nullptr;
-}
-
 // }}}
 // {{{ definition of Aggregate::simplify
 
@@ -1075,7 +1059,6 @@ bool Conjunction::check(ChkLvlVec &levels) const {
     return ret;
 }
 bool SimpleBodyLiteral::check(ChkLvlVec &levels) const { 
-    levels.back().current = &levels.back().dep.insertEnt();
     _add(levels, lit, true);
     return true;
 }
