@@ -23,7 +23,9 @@
 
 namespace Gringo {
 
-Scripts::Scripts() = default;
+Scripts::Scripts(GringoModule &module) 
+    : py(module)
+    , lua(module) { }
 
 bool Scripts::luaExec(Location const &loc, FWString code) {
     return lua.exec(loc, code);
@@ -42,10 +44,11 @@ void Scripts::main(Control &ctl) {
 ValVec Scripts::call(Location const &loc, FWString name, ValVec const &args) {
     if (py.callable(context, name)) { return py.call(context, loc, name, args); }
     if (lua.callable(context, name)) { return lua.call(context, loc, name, args); }
-    GRINGO_REPORT(W_TERM_UNDEFINED)
-        << loc << ": warning: function not found, a zero is substituted\n"
+    GRINGO_REPORT(W_OPERATION_UNDEFINED)
+        << loc << ": info: operation undefined:\n"
+        << "  function '" << *name << "' not found\n"
         ;
-    return {0};
+    return {};
 }
 Scripts::~Scripts() = default;
 
