@@ -1039,6 +1039,18 @@ void TestLparse::test_disjoint() {
             "35 $<= $y $<= 36.\n"
             "not #disjoint{ 1:6$*$y; 2:35$*$x }.\n"
             )));
+    CPPUNIT_ASSERT_EQUAL(
+        576,
+        (int)solve(
+            "#const n = 4.\n"
+            "row(1..n).\n"
+            "col(1..n).\n"
+            "dom(1,n).\n"
+            "\n"
+            "L $<= $cell(X,Y) $<= U :- row(X), col(Y), dom(L,U).\n"
+            ":- col(Y), not #disjoint { X : $cell(X,Y) : row(X) }.\n"
+            ":- row(X), not #disjoint { Y : $cell(X,Y) : col(Y) }.\n"
+            ).size());
 }
 
 void TestLparse::test_queens() {
