@@ -22,6 +22,7 @@
 
 #include "tests/tests.hh"
 #include "gringo/python.hh"
+#include "gringo_module.hh"
 
 namespace Gringo { namespace Test {
 
@@ -29,6 +30,7 @@ namespace Gringo { namespace Test {
 
 class TestPython : public CppUnit::TestFixture {
     CPPUNIT_TEST_SUITE(TestPython);
+        CPPUNIT_TEST(test_parse);
         CPPUNIT_TEST(test_callable);
         CPPUNIT_TEST(test_values);
         CPPUNIT_TEST(test_cmp);
@@ -38,6 +40,7 @@ public:
     virtual void setUp();
     virtual void tearDown();
 
+    void test_parse();
     void test_values();
     void test_cmp();
     void test_callable();
@@ -68,9 +71,23 @@ std::string replace(std::string &&x, std::string const &y, std::string const &z)
     }
     return std::move(x);
 }
+void TestPython::test_parse() {
+    Location loc("dummy", 1, 1, "dummy", 1, 1);
+    Python py(getTestModule());
+    py.exec(loc,
+        "import gringo\n"
+        "def get(): return gringo.parse_term('1')\n"
+        );
+    CPPUNIT_ASSERT_EQUAL(S("[1]"), to_string(py.call(Any(), loc, "get", {})));
+    py.exec(loc,
+        "import gringo\n"
+        "def get(): return gringo.parse_term('p(1+2)')\n"
+        );
+    CPPUNIT_ASSERT_EQUAL(S("[p(3)]"), to_string(py.call(Any(), loc, "get", {})));
+}
 void TestPython::test_values() {
     Location loc("dummy", 1, 1, "dummy", 1, 1);
-    Python py;
+    Python py(getTestModule());
     py.exec(loc,
         "import gringo\n"
         "x = gringo.Fun(\"f\", [2, 3, 4])\n"
@@ -79,8 +96,8 @@ void TestPython::test_values() {
         "def none(): return None\n"
         "values = ["
         "gringo.Fun(\"f\", [1, 2, 3]),"
-        "gringo.Sup(),"
-        "gringo.Inf(),"
+        "gringo.Sup,"
+        "gringo.Inf,"
         "gringo.Fun(\"id\"),"
         "(1, 2, 3),"
         "123,"
@@ -94,19 +111,19 @@ void TestPython::test_values() {
     CPPUNIT_ASSERT_EQUAL(S("[f(1,2,3),#sup,#inf,id,(1,2,3),123,\"abc\",(2,3,4),\"f\"]"), to_string(py.call(Any(), loc, "getValues", {})));
     {
         Gringo::Test::Messages msg;
-        CPPUNIT_ASSERT_EQUAL(S("[0]"), to_string(py.call(Any(), loc, "none", {})));
+        CPPUNIT_ASSERT_EQUAL(S("[]"), to_string(py.call(Any(), loc, "none", {})));
         CPPUNIT_ASSERT_EQUAL(S(
             "["
-            "dummy:1:1: warning: operation undefined, a zero is substituted:\n"
+            "dummy:1:1: info: operation undefined:\n"
             "  RuntimeError: cannot convert to value: unexpected NoneType() object\n"
             "]"), IO::to_string(msg));
     }
     {
         Gringo::Test::Messages msg;
-        CPPUNIT_ASSERT_EQUAL(S("[0]"), to_string(py.call(Any(), loc, "fail", {})));
+        CPPUNIT_ASSERT_EQUAL(S("[]"), to_string(py.call(Any(), loc, "fail", {})));
         CPPUNIT_ASSERT_EQUAL(S(
             "["
-            "dummy:1:1: warning: operation undefined, a zero is substituted:\n"
+            "dummy:1:1: info: operation undefined:\n"
             "  Traceback (most recent call last):\n"
             "    File \"<dummy:1:1>\", line 4, in fail\n"
             "  RuntimeError: cannot convert to value: unexpected NoneType() object\n"
@@ -114,10 +131,10 @@ void TestPython::test_values() {
     }
     {
         Gringo::Test::Messages msg;
-        py.exec(loc, "(");
+        CPPUNIT_ASSERT_THROW(py.exec(loc, "("), std::runtime_error);
         CPPUNIT_ASSERT_EQUAL(S(
             "["
-            "dummy:1:1: warning: parsing failed:\n"
+            "dummy:1:1: error: parsing failed:\n"
             "    File \"<dummy:1:1>\", line 1\n"
             "      (\n"
             "      ^\n"
@@ -128,7 +145,7 @@ void TestPython::test_values() {
 
 void TestPython::test_cmp() {
     Location loc("dummy", 1, 1, "dummy", 1, 1);
-    Python py;
+    Python py(getTestModule());
     py.exec(loc, 
         "import gringo\n"
         "def cmp():\n"
@@ -144,7 +161,7 @@ void TestPython::test_cmp() {
 
 void TestPython::test_callable() {
     Location loc("dummy", 1, 1, "dummy", 1, 1);
-    Python py;
+    Python py(getTestModule());
     py.exec(loc, 
         "import gringo\n"
         "def a(): pass\n"
